"""Generated CLI commands for Tags tag (auto-generated, do not edit)."""

from __future__ import annotations

import typer
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from immich import AsyncClient

from immich.cli.runtime import print_response, run_command, set_nested
from immich.client.generated.models import *

app = typer.Typer(
    help="""A tag is a user-defined label that can be applied to assets for organizational purposes. Tags can also be hierarchical, allowing for parent-child relationships between tags.\n\n[link=https://api.immich.app/endpoints/tags]Immich API documentation[/link]"""
)


@app.command("bulk-tag-assets", deprecated=False, rich_help_panel="API commands")
def bulk_tag_assets(
    ctx: typer.Context,
    asset_ids: list[str] = typer.Option(..., "--asset-ids", help=""""""),
    tag_ids: list[str] = typer.Option(..., "--tag-ids", help=""""""),
) -> None:
    """Tag assets

    [link=https://api.immich.app/endpoints/tags/bulkTagAssets]Immich API documentation[/link]
    """
    kwargs = {}
    json_data = {}
    set_nested(json_data, ["asset_ids"], asset_ids)
    set_nested(json_data, ["tag_ids"], tag_ids)
    tag_bulk_assets_dto = TagBulkAssetsDto.model_validate(json_data)
    kwargs["tag_bulk_assets_dto"] = tag_bulk_assets_dto
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.tags, "bulk_tag_assets", ctx, **kwargs)
    print_response(result, ctx)


@app.command("create-tag", deprecated=False, rich_help_panel="API commands")
def create_tag(
    ctx: typer.Context,
    color: str | None = typer.Option(None, "--color", help=""""""),
    name: str = typer.Option(..., "--name", help=""""""),
    parent_id: str | None = typer.Option(None, "--parent-id", help=""""""),
) -> None:
    """Create a tag

    [link=https://api.immich.app/endpoints/tags/createTag]Immich API documentation[/link]
    """
    kwargs = {}
    json_data = {}
    if color is not None:
        set_nested(json_data, ["color"], color)
    set_nested(json_data, ["name"], name)
    if parent_id is not None:
        set_nested(json_data, ["parent_id"], parent_id)
    tag_create_dto = TagCreateDto.model_validate(json_data)
    kwargs["tag_create_dto"] = tag_create_dto
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.tags, "create_tag", ctx, **kwargs)
    print_response(result, ctx)


@app.command("delete-tag", deprecated=False, rich_help_panel="API commands")
def delete_tag(
    ctx: typer.Context,
    id: str = typer.Argument(..., help=""""""),
) -> None:
    """Delete a tag

    [link=https://api.immich.app/endpoints/tags/deleteTag]Immich API documentation[/link]
    """
    kwargs = {}
    kwargs["id"] = id
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.tags, "delete_tag", ctx, **kwargs)
    print_response(result, ctx)


@app.command("get-all-tags", deprecated=False, rich_help_panel="API commands")
def get_all_tags(
    ctx: typer.Context,
) -> None:
    """Retrieve tags

    [link=https://api.immich.app/endpoints/tags/getAllTags]Immich API documentation[/link]
    """
    kwargs = {}
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.tags, "get_all_tags", ctx, **kwargs)
    print_response(result, ctx)


@app.command("get-tag-by-id", deprecated=False, rich_help_panel="API commands")
def get_tag_by_id(
    ctx: typer.Context,
    id: str = typer.Argument(..., help=""""""),
) -> None:
    """Retrieve a tag

    [link=https://api.immich.app/endpoints/tags/getTagById]Immich API documentation[/link]
    """
    kwargs = {}
    kwargs["id"] = id
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.tags, "get_tag_by_id", ctx, **kwargs)
    print_response(result, ctx)


@app.command("tag-assets", deprecated=False, rich_help_panel="API commands")
def tag_assets(
    ctx: typer.Context,
    id: str = typer.Argument(..., help=""""""),
    ids: list[str] = typer.Option(..., "--ids", help=""""""),
) -> None:
    """Tag assets

    [link=https://api.immich.app/endpoints/tags/tagAssets]Immich API documentation[/link]
    """
    kwargs = {}
    json_data = {}
    kwargs["id"] = id
    set_nested(json_data, ["ids"], ids)
    bulk_ids_dto = BulkIdsDto.model_validate(json_data)
    kwargs["bulk_ids_dto"] = bulk_ids_dto
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.tags, "tag_assets", ctx, **kwargs)
    print_response(result, ctx)


@app.command("untag-assets", deprecated=False, rich_help_panel="API commands")
def untag_assets(
    ctx: typer.Context,
    id: str = typer.Argument(..., help=""""""),
    ids: list[str] = typer.Option(..., "--ids", help=""""""),
) -> None:
    """Untag assets

    [link=https://api.immich.app/endpoints/tags/untagAssets]Immich API documentation[/link]
    """
    kwargs = {}
    json_data = {}
    kwargs["id"] = id
    set_nested(json_data, ["ids"], ids)
    bulk_ids_dto = BulkIdsDto.model_validate(json_data)
    kwargs["bulk_ids_dto"] = bulk_ids_dto
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.tags, "untag_assets", ctx, **kwargs)
    print_response(result, ctx)


@app.command("update-tag", deprecated=False, rich_help_panel="API commands")
def update_tag(
    ctx: typer.Context,
    id: str = typer.Argument(..., help=""""""),
    color: str | None = typer.Option(None, "--color", help=""""""),
) -> None:
    """Update a tag

    [link=https://api.immich.app/endpoints/tags/updateTag]Immich API documentation[/link]
    """
    kwargs = {}
    json_data = {}
    kwargs["id"] = id
    if color is not None:
        set_nested(json_data, ["color"], color)
    tag_update_dto = TagUpdateDto.model_validate(json_data)
    kwargs["tag_update_dto"] = tag_update_dto
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.tags, "update_tag", ctx, **kwargs)
    print_response(result, ctx)


@app.command("upsert-tags", deprecated=False, rich_help_panel="API commands")
def upsert_tags(
    ctx: typer.Context,
    tags: list[str] = typer.Option(..., "--tags", help=""""""),
) -> None:
    """Upsert tags

    [link=https://api.immich.app/endpoints/tags/upsertTags]Immich API documentation[/link]
    """
    kwargs = {}
    json_data = {}
    set_nested(json_data, ["tags"], tags)
    tag_upsert_dto = TagUpsertDto.model_validate(json_data)
    kwargs["tag_upsert_dto"] = tag_upsert_dto
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.tags, "upsert_tags", ctx, **kwargs)
    print_response(result, ctx)



from pathlib import Path
from typing import Literal, Optional, Union
from multidict import CIMultiDictProxy
from pydantic import BaseModel, Field

from immich.client.generated import AssetMediaResponseDto

HeadersType = Union[dict[str, str], CIMultiDictProxy[str]]
RejectionReason = Literal["duplicate", "unsupported_format"]


class UploadStats(BaseModel):
    total: int = Field(..., description="The total number of files to upload.")
    uploaded: int = Field(..., description="The number of files that were uploaded.")
    rejected: int = Field(..., description="The number of files that were rejected.")
    failed: int = Field(..., description="The number of files that failed to upload.")


class RejectedEntry(BaseModel):
    """Represents a file that was rejected during upload (check)."""

    filepath: Path = Field(
        ..., description="The path to the local file that was rejected."
    )
    asset_id: Optional[str] = Field(
        None, description="The ID of the asset. Set if reason is 'duplicate'."
    )
    reason: Optional[RejectionReason] = Field(
        None, description="The reason for the rejection."
    )


class FailedEntry(BaseModel):
    """Represents a file that failed to upload."""

    filepath: Path = Field(
        ..., description="The path to the local file that failed to upload."
    )
    error: str = Field(..., description="The error message from the server.")


class UploadedEntry(BaseModel):
    """Represents a successfully uploaded file."""

    asset: AssetMediaResponseDto = Field(
        ..., description="The asset that was uploaded."
    )
    filepath: Path = Field(
        ..., description="The path to the local file that was uploaded."
    )


class UploadResult(BaseModel):
    """The result of an upload operation containing all uploaded, rejected, and failed entries."""

    uploaded: list[UploadedEntry] = Field(
        ..., description="The assets that were uploaded."
    )
    rejected: list[RejectedEntry] = Field(
        ..., description="The files that were rejected."
    )
    failed: list[FailedEntry] = Field(
        ..., description="The files that failed to upload."
    )
    stats: UploadStats = Field(..., description="The statistics of the upload.")



"""Generated CLI commands for Assets tag (auto-generated, do not edit)."""

from __future__ import annotations

import typer
import json
from datetime import datetime
from pathlib import Path
from typing import Literal, TYPE_CHECKING

if TYPE_CHECKING:
    from immich import AsyncClient

from immich.cli.runtime import print_response, run_command, set_nested
from immich.client.generated.models import *

app = typer.Typer(
    help="""An asset is an image or video that has been uploaded to Immich.\n\n[link=https://api.immich.app/endpoints/assets]Immich API documentation[/link]"""
)


@app.command("check-bulk-upload", deprecated=False, rich_help_panel="API commands")
def check_bulk_upload(
    ctx: typer.Context,
    assets: list[str] = typer.Option(..., "--assets", help="""As a JSON string"""),
) -> None:
    """Check bulk upload

    [link=https://api.immich.app/endpoints/assets/checkBulkUpload]Immich API documentation[/link]
    """
    kwargs = {}
    json_data = {}
    value_assets = [json.loads(i) for i in assets]
    set_nested(json_data, ["assets"], value_assets)
    asset_bulk_upload_check_dto = AssetBulkUploadCheckDto.model_validate(json_data)
    kwargs["asset_bulk_upload_check_dto"] = asset_bulk_upload_check_dto
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.assets, "check_bulk_upload", ctx, **kwargs)
    print_response(result, ctx)


@app.command("check-existing-assets", deprecated=False, rich_help_panel="API commands")
def check_existing_assets(
    ctx: typer.Context,
    device_asset_ids: list[str] = typer.Option(..., "--device-asset-ids", help=""""""),
    device_id: str = typer.Option(..., "--device-id", help=""""""),
) -> None:
    """Check existing assets

    [link=https://api.immich.app/endpoints/assets/checkExistingAssets]Immich API documentation[/link]
    """
    kwargs = {}
    json_data = {}
    set_nested(json_data, ["device_asset_ids"], device_asset_ids)
    set_nested(json_data, ["device_id"], device_id)
    check_existing_assets_dto = CheckExistingAssetsDto.model_validate(json_data)
    kwargs["check_existing_assets_dto"] = check_existing_assets_dto
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.assets, "check_existing_assets", ctx, **kwargs)
    print_response(result, ctx)


@app.command("copy-asset", deprecated=False, rich_help_panel="API commands")
def copy_asset(
    ctx: typer.Context,
    albums: Literal["true", "false"] | None = typer.Option(
        None, "--albums", help=""""""
    ),
    favorite: Literal["true", "false"] | None = typer.Option(
        None, "--favorite", help=""""""
    ),
    shared_links: Literal["true", "false"] | None = typer.Option(
        None, "--shared-links", help=""""""
    ),
    sidecar: Literal["true", "false"] | None = typer.Option(
        None, "--sidecar", help=""""""
    ),
    source_id: str = typer.Option(..., "--source-id", help=""""""),
    stack: Literal["true", "false"] | None = typer.Option(None, "--stack", help=""""""),
    target_id: str = typer.Option(..., "--target-id", help=""""""),
) -> None:
    """Copy asset

    [link=https://api.immich.app/endpoints/assets/copyAsset]Immich API documentation[/link]
    """
    kwargs = {}
    json_data = {}
    if albums is not None:
        set_nested(json_data, ["albums"], albums.lower() == "true")
    if favorite is not None:
        set_nested(json_data, ["favorite"], favorite.lower() == "true")
    if shared_links is not None:
        set_nested(json_data, ["shared_links"], shared_links.lower() == "true")
    if sidecar is not None:
        set_nested(json_data, ["sidecar"], sidecar.lower() == "true")
    set_nested(json_data, ["source_id"], source_id)
    if stack is not None:
        set_nested(json_data, ["stack"], stack.lower() == "true")
    set_nested(json_data, ["target_id"], target_id)
    asset_copy_dto = AssetCopyDto.model_validate(json_data)
    kwargs["asset_copy_dto"] = asset_copy_dto
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.assets, "copy_asset", ctx, **kwargs)
    print_response(result, ctx)


@app.command("delete-asset-metadata", deprecated=False, rich_help_panel="API commands")
def delete_asset_metadata(
    ctx: typer.Context,
    id: str = typer.Argument(..., help=""""""),
    key: str = typer.Argument(..., help=""""""),
) -> None:
    """Delete asset metadata by key

    [link=https://api.immich.app/endpoints/assets/deleteAssetMetadata]Immich API documentation[/link]
    """
    kwargs = {}
    kwargs["id"] = id
    kwargs["key"] = key
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.assets, "delete_asset_metadata", ctx, **kwargs)
    print_response(result, ctx)


@app.command("delete-assets", deprecated=False, rich_help_panel="API commands")
def delete_assets(
    ctx: typer.Context,
    force: Literal["true", "false"] | None = typer.Option(None, "--force", help=""""""),
    ids: list[str] = typer.Option(..., "--ids", help=""""""),
) -> None:
    """Delete assets

    [link=https://api.immich.app/endpoints/assets/deleteAssets]Immich API documentation[/link]
    """
    kwargs = {}
    json_data = {}
    if force is not None:
        set_nested(json_data, ["force"], force.lower() == "true")
    set_nested(json_data, ["ids"], ids)
    asset_bulk_delete_dto = AssetBulkDeleteDto.model_validate(json_data)
    kwargs["asset_bulk_delete_dto"] = asset_bulk_delete_dto
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.assets, "delete_assets", ctx, **kwargs)
    print_response(result, ctx)


@app.command(
    "delete-bulk-asset-metadata", deprecated=False, rich_help_panel="API commands"
)
def delete_bulk_asset_metadata(
    ctx: typer.Context,
    items: list[str] = typer.Option(..., "--items", help="""As a JSON string"""),
) -> None:
    """Delete asset metadata

    [link=https://api.immich.app/endpoints/assets/deleteBulkAssetMetadata]Immich API documentation[/link]
    """
    kwargs = {}
    json_data = {}
    value_items = [json.loads(i) for i in items]
    set_nested(json_data, ["items"], value_items)
    asset_metadata_bulk_delete_dto = AssetMetadataBulkDeleteDto.model_validate(
        json_data
    )
    kwargs["asset_metadata_bulk_delete_dto"] = asset_metadata_bulk_delete_dto
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(
        client, client.assets, "delete_bulk_asset_metadata", ctx, **kwargs
    )
    print_response(result, ctx)


@app.command("download-asset", deprecated=False, rich_help_panel="API commands")
def download_asset(
    ctx: typer.Context,
    id: str = typer.Argument(..., help=""""""),
    edited: Literal["true", "false"] | None = typer.Option(
        None, "--edited", help=""""""
    ),
    key: str | None = typer.Option(None, "--key", help=""""""),
    slug: str | None = typer.Option(None, "--slug", help=""""""),
) -> None:
    """Download original asset

    [link=https://api.immich.app/endpoints/assets/downloadAsset]Immich API documentation[/link]
    """
    kwargs = {}
    if edited is not None:
        kwargs["edited"] = edited.lower() == "true"
    kwargs["id"] = id
    if key is not None:
        kwargs["key"] = key
    if slug is not None:
        kwargs["slug"] = slug
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.assets, "download_asset", ctx, **kwargs)
    print_response(result, ctx)


@app.command("edit-asset", deprecated=False, rich_help_panel="API commands")
def edit_asset(
    ctx: typer.Context,
    id: str = typer.Argument(..., help=""""""),
    edits: list[str] = typer.Option(
        ...,
        "--edits",
        help="""list of edits

As a JSON string""",
    ),
) -> None:
    """Apply edits to an existing asset

    [link=https://api.immich.app/endpoints/assets/editAsset]Immich API documentation[/link]
    """
    kwargs = {}
    json_data = {}
    kwargs["id"] = id
    value_edits = [json.loads(i) for i in edits]
    set_nested(json_data, ["edits"], value_edits)
    asset_edit_action_list_dto = AssetEditActionListDto.model_validate(json_data)
    kwargs["asset_edit_action_list_dto"] = asset_edit_action_list_dto
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.assets, "edit_asset", ctx, **kwargs)
    print_response(result, ctx)


@app.command(
    "get-all-user-assets-by-device-id", deprecated=True, rich_help_panel="API commands"
)
def get_all_user_assets_by_device_id(
    ctx: typer.Context,
    device_id: str = typer.Argument(..., help=""""""),
) -> None:
    """Retrieve assets by device ID

    [link=https://api.immich.app/endpoints/assets/getAllUserAssetsByDeviceId]Immich API documentation[/link]
    """
    kwargs = {}
    kwargs["device_id"] = device_id
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(
        client, client.assets, "get_all_user_assets_by_device_id", ctx, **kwargs
    )
    print_response(result, ctx)


@app.command("get-asset-edits", deprecated=False, rich_help_panel="API commands")
def get_asset_edits(
    ctx: typer.Context,
    id: str = typer.Argument(..., help=""""""),
) -> None:
    """Retrieve edits for an existing asset

    [link=https://api.immich.app/endpoints/assets/getAssetEdits]Immich API documentation[/link]
    """
    kwargs = {}
    kwargs["id"] = id
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.assets, "get_asset_edits", ctx, **kwargs)
    print_response(result, ctx)


@app.command("get-asset-info", deprecated=False, rich_help_panel="API commands")
def get_asset_info(
    ctx: typer.Context,
    id: str = typer.Argument(..., help=""""""),
    key: str | None = typer.Option(None, "--key", help=""""""),
    slug: str | None = typer.Option(None, "--slug", help=""""""),
) -> None:
    """Retrieve an asset

    [link=https://api.immich.app/endpoints/assets/getAssetInfo]Immich API documentation[/link]
    """
    kwargs = {}
    kwargs["id"] = id
    if key is not None:
        kwargs["key"] = key
    if slug is not None:
        kwargs["slug"] = slug
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.assets, "get_asset_info", ctx, **kwargs)
    print_response(result, ctx)


@app.command("get-asset-metadata", deprecated=False, rich_help_panel="API commands")
def get_asset_metadata(
    ctx: typer.Context,
    id: str = typer.Argument(..., help=""""""),
) -> None:
    """Get asset metadata

    [link=https://api.immich.app/endpoints/assets/getAssetMetadata]Immich API documentation[/link]
    """
    kwargs = {}
    kwargs["id"] = id
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.assets, "get_asset_metadata", ctx, **kwargs)
    print_response(result, ctx)


@app.command(
    "get-asset-metadata-by-key", deprecated=False, rich_help_panel="API commands"
)
def get_asset_metadata_by_key(
    ctx: typer.Context,
    id: str = typer.Argument(..., help=""""""),
    key: str = typer.Argument(..., help=""""""),
) -> None:
    """Retrieve asset metadata by key

    [link=https://api.immich.app/endpoints/assets/getAssetMetadataByKey]Immich API documentation[/link]
    """
    kwargs = {}
    kwargs["id"] = id
    kwargs["key"] = key
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(
        client, client.assets, "get_asset_metadata_by_key", ctx, **kwargs
    )
    print_response(result, ctx)


@app.command("get-asset-ocr", deprecated=False, rich_help_panel="API commands")
def get_asset_ocr(
    ctx: typer.Context,
    id: str = typer.Argument(..., help=""""""),
) -> None:
    """Retrieve asset OCR data

    [link=https://api.immich.app/endpoints/assets/getAssetOcr]Immich API documentation[/link]
    """
    kwargs = {}
    kwargs["id"] = id
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.assets, "get_asset_ocr", ctx, **kwargs)
    print_response(result, ctx)


@app.command("get-asset-statistics", deprecated=False, rich_help_panel="API commands")
def get_asset_statistics(
    ctx: typer.Context,
    is_favorite: Literal["true", "false"] | None = typer.Option(
        None, "--is-favorite", help=""""""
    ),
    is_trashed: Literal["true", "false"] | None = typer.Option(
        None, "--is-trashed", help=""""""
    ),
    visibility: AssetVisibility | None = typer.Option(
        None, "--visibility", help=""""""
    ),
) -> None:
    """Get asset statistics

    [link=https://api.immich.app/endpoints/assets/getAssetStatistics]Immich API documentation[/link]
    """
    kwargs = {}
    if is_favorite is not None:
        kwargs["is_favorite"] = is_favorite.lower() == "true"
    if is_trashed is not None:
        kwargs["is_trashed"] = is_trashed.lower() == "true"
    if visibility is not None:
        kwargs["visibility"] = visibility
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.assets, "get_asset_statistics", ctx, **kwargs)
    print_response(result, ctx)


@app.command("get-random", deprecated=True, rich_help_panel="API commands")
def get_random(
    ctx: typer.Context,
    count: float | None = typer.Option(None, "--count", help="""""", min=1),
) -> None:
    """Get random assets

    [link=https://api.immich.app/endpoints/assets/getRandom]Immich API documentation[/link]
    """
    kwargs = {}
    if count is not None:
        kwargs["count"] = count
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.assets, "get_random", ctx, **kwargs)
    print_response(result, ctx)


@app.command("play-asset-video", deprecated=False, rich_help_panel="API commands")
def play_asset_video(
    ctx: typer.Context,
    id: str = typer.Argument(..., help=""""""),
    key: str | None = typer.Option(None, "--key", help=""""""),
    slug: str | None = typer.Option(None, "--slug", help=""""""),
) -> None:
    """Play asset video

    [link=https://api.immich.app/endpoints/assets/playAssetVideo]Immich API documentation[/link]
    """
    kwargs = {}
    kwargs["id"] = id
    if key is not None:
        kwargs["key"] = key
    if slug is not None:
        kwargs["slug"] = slug
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.assets, "play_asset_video", ctx, **kwargs)
    print_response(result, ctx)


@app.command("remove-asset-edits", deprecated=False, rich_help_panel="API commands")
def remove_asset_edits(
    ctx: typer.Context,
    id: str = typer.Argument(..., help=""""""),
) -> None:
    """Remove edits from an existing asset

    [link=https://api.immich.app/endpoints/assets/removeAssetEdits]Immich API documentation[/link]
    """
    kwargs = {}
    kwargs["id"] = id
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.assets, "remove_asset_edits", ctx, **kwargs)
    print_response(result, ctx)


@app.command("replace-asset", deprecated=True, rich_help_panel="API commands")
def replace_asset(
    ctx: typer.Context,
    id: str = typer.Argument(..., help=""""""),
    asset_data: Path = typer.Option(..., "--asset-data", help="""""", exists=True),
    device_asset_id: str = typer.Option(..., "--device-asset-id", help=""""""),
    device_id: str = typer.Option(..., "--device-id", help=""""""),
    duration: str | None = typer.Option(None, "--duration", help=""""""),
    file_created_at: datetime = typer.Option(..., "--file-created-at", help=""""""),
    file_modified_at: datetime = typer.Option(..., "--file-modified-at", help=""""""),
    filename: str | None = typer.Option(None, "--filename", help=""""""),
    key: str | None = typer.Option(None, "--key", help=""""""),
    slug: str | None = typer.Option(None, "--slug", help=""""""),
) -> None:
    """Replace asset

    [link=https://api.immich.app/endpoints/assets/replaceAsset]Immich API documentation[/link]
    """
    kwargs = {}
    json_data = {}
    kwargs["id"] = id
    if key is not None:
        kwargs["key"] = key
    if slug is not None:
        kwargs["slug"] = slug
    kwargs["asset_data"] = (asset_data.name, asset_data.read_bytes())
    set_nested(json_data, ["device_asset_id"], device_asset_id)
    set_nested(json_data, ["device_id"], device_id)
    if duration is not None:
        set_nested(json_data, ["duration"], duration)
    set_nested(json_data, ["file_created_at"], file_created_at)
    set_nested(json_data, ["file_modified_at"], file_modified_at)
    if filename is not None:
        set_nested(json_data, ["filename"], filename)
    kwargs.update(json_data)
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.assets, "replace_asset", ctx, **kwargs)
    print_response(result, ctx)


@app.command("run-asset-jobs", deprecated=False, rich_help_panel="API commands")
def run_asset_jobs(
    ctx: typer.Context,
    asset_ids: list[str] = typer.Option(..., "--asset-ids", help=""""""),
    name: str = typer.Option(..., "--name", help=""""""),
) -> None:
    """Run an asset job

    [link=https://api.immich.app/endpoints/assets/runAssetJobs]Immich API documentation[/link]
    """
    kwargs = {}
    json_data = {}
    set_nested(json_data, ["asset_ids"], asset_ids)
    set_nested(json_data, ["name"], name)
    asset_jobs_dto = AssetJobsDto.model_validate(json_data)
    kwargs["asset_jobs_dto"] = asset_jobs_dto
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.assets, "run_asset_jobs", ctx, **kwargs)
    print_response(result, ctx)


@app.command("update-asset", deprecated=False, rich_help_panel="API commands")
def update_asset(
    ctx: typer.Context,
    id: str = typer.Argument(..., help=""""""),
    date_time_original: str | None = typer.Option(
        None, "--date-time-original", help=""""""
    ),
    description: str | None = typer.Option(None, "--description", help=""""""),
    is_favorite: Literal["true", "false"] | None = typer.Option(
        None, "--is-favorite", help=""""""
    ),
    latitude: float | None = typer.Option(None, "--latitude", help=""""""),
    live_photo_video_id: str | None = typer.Option(
        None, "--live-photo-video-id", help=""""""
    ),
    longitude: float | None = typer.Option(None, "--longitude", help=""""""),
    rating: float | None = typer.Option(None, "--rating", help="""""", min=-1, max=5),
    visibility: str | None = typer.Option(None, "--visibility", help=""""""),
) -> None:
    """Update an asset

    [link=https://api.immich.app/endpoints/assets/updateAsset]Immich API documentation[/link]
    """
    kwargs = {}
    json_data = {}
    kwargs["id"] = id
    if date_time_original is not None:
        set_nested(json_data, ["date_time_original"], date_time_original)
    if description is not None:
        set_nested(json_data, ["description"], description)
    if is_favorite is not None:
        set_nested(json_data, ["is_favorite"], is_favorite.lower() == "true")
    if latitude is not None:
        set_nested(json_data, ["latitude"], latitude)
    if live_photo_video_id is not None:
        set_nested(json_data, ["live_photo_video_id"], live_photo_video_id)
    if longitude is not None:
        set_nested(json_data, ["longitude"], longitude)
    if rating is not None:
        set_nested(json_data, ["rating"], rating)
    if visibility is not None:
        set_nested(json_data, ["visibility"], visibility)
    update_asset_dto = UpdateAssetDto.model_validate(json_data)
    kwargs["update_asset_dto"] = update_asset_dto
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.assets, "update_asset", ctx, **kwargs)
    print_response(result, ctx)


@app.command("update-asset-metadata", deprecated=False, rich_help_panel="API commands")
def update_asset_metadata(
    ctx: typer.Context,
    id: str = typer.Argument(..., help=""""""),
    items: list[str] = typer.Option(..., "--items", help="""As a JSON string"""),
) -> None:
    """Update asset metadata

    [link=https://api.immich.app/endpoints/assets/updateAssetMetadata]Immich API documentation[/link]
    """
    kwargs = {}
    json_data = {}
    kwargs["id"] = id
    value_items = [json.loads(i) for i in items]
    set_nested(json_data, ["items"], value_items)
    asset_metadata_upsert_dto = AssetMetadataUpsertDto.model_validate(json_data)
    kwargs["asset_metadata_upsert_dto"] = asset_metadata_upsert_dto
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.assets, "update_asset_metadata", ctx, **kwargs)
    print_response(result, ctx)


@app.command("update-assets", deprecated=False, rich_help_panel="API commands")
def update_assets(
    ctx: typer.Context,
    date_time_original: str | None = typer.Option(
        None, "--date-time-original", help=""""""
    ),
    date_time_relative: float | None = typer.Option(
        None, "--date-time-relative", help=""""""
    ),
    description: str | None = typer.Option(None, "--description", help=""""""),
    duplicate_id: str | None = typer.Option(None, "--duplicate-id", help=""""""),
    ids: list[str] = typer.Option(..., "--ids", help=""""""),
    is_favorite: Literal["true", "false"] | None = typer.Option(
        None, "--is-favorite", help=""""""
    ),
    latitude: float | None = typer.Option(None, "--latitude", help=""""""),
    longitude: float | None = typer.Option(None, "--longitude", help=""""""),
    rating: float | None = typer.Option(None, "--rating", help="""""", min=-1, max=5),
    time_zone: str | None = typer.Option(None, "--time-zone", help=""""""),
    visibility: str | None = typer.Option(None, "--visibility", help=""""""),
) -> None:
    """Update assets

    [link=https://api.immich.app/endpoints/assets/updateAssets]Immich API documentation[/link]
    """
    kwargs = {}
    json_data = {}
    if date_time_original is not None:
        set_nested(json_data, ["date_time_original"], date_time_original)
    if date_time_relative is not None:
        set_nested(json_data, ["date_time_relative"], date_time_relative)
    if description is not None:
        set_nested(json_data, ["description"], description)
    if duplicate_id is not None:
        set_nested(json_data, ["duplicate_id"], duplicate_id)
    set_nested(json_data, ["ids"], ids)
    if is_favorite is not None:
        set_nested(json_data, ["is_favorite"], is_favorite.lower() == "true")
    if latitude is not None:
        set_nested(json_data, ["latitude"], latitude)
    if longitude is not None:
        set_nested(json_data, ["longitude"], longitude)
    if rating is not None:
        set_nested(json_data, ["rating"], rating)
    if time_zone is not None:
        set_nested(json_data, ["time_zone"], time_zone)
    if visibility is not None:
        set_nested(json_data, ["visibility"], visibility)
    asset_bulk_update_dto = AssetBulkUpdateDto.model_validate(json_data)
    kwargs["asset_bulk_update_dto"] = asset_bulk_update_dto
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.assets, "update_assets", ctx, **kwargs)
    print_response(result, ctx)


@app.command(
    "update-bulk-asset-metadata", deprecated=False, rich_help_panel="API commands"
)
def update_bulk_asset_metadata(
    ctx: typer.Context,
    items: list[str] = typer.Option(..., "--items", help="""As a JSON string"""),
) -> None:
    """Upsert asset metadata

    [link=https://api.immich.app/endpoints/assets/updateBulkAssetMetadata]Immich API documentation[/link]
    """
    kwargs = {}
    json_data = {}
    value_items = [json.loads(i) for i in items]
    set_nested(json_data, ["items"], value_items)
    asset_metadata_bulk_upsert_dto = AssetMetadataBulkUpsertDto.model_validate(
        json_data
    )
    kwargs["asset_metadata_bulk_upsert_dto"] = asset_metadata_bulk_upsert_dto
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(
        client, client.assets, "update_bulk_asset_metadata", ctx, **kwargs
    )
    print_response(result, ctx)


@app.command("upload-asset", deprecated=False, rich_help_panel="API commands")
def upload_asset(
    ctx: typer.Context,
    asset_data: Path = typer.Option(..., "--asset-data", help="""""", exists=True),
    device_asset_id: str = typer.Option(..., "--device-asset-id", help=""""""),
    device_id: str = typer.Option(..., "--device-id", help=""""""),
    duration: str | None = typer.Option(None, "--duration", help=""""""),
    file_created_at: datetime = typer.Option(..., "--file-created-at", help=""""""),
    file_modified_at: datetime = typer.Option(..., "--file-modified-at", help=""""""),
    filename: str | None = typer.Option(None, "--filename", help=""""""),
    is_favorite: Literal["true", "false"] | None = typer.Option(
        None, "--is-favorite", help=""""""
    ),
    key: str | None = typer.Option(None, "--key", help=""""""),
    live_photo_video_id: str | None = typer.Option(
        None, "--live-photo-video-id", help=""""""
    ),
    metadata: list[str] | None = typer.Option(
        None, "--metadata", help="""As a JSON string"""
    ),
    sidecar_data: Path | None = typer.Option(
        None, "--sidecar-data", help="""""", exists=True
    ),
    slug: str | None = typer.Option(None, "--slug", help=""""""),
    visibility: str | None = typer.Option(None, "--visibility", help=""""""),
    x_immich_checksum: str | None = typer.Option(
        None,
        "--x-immich-checksum",
        help="""sha1 checksum that can be used for duplicate detection before the file is uploaded""",
    ),
) -> None:
    """Upload asset

    [link=https://api.immich.app/endpoints/assets/uploadAsset]Immich API documentation[/link]
    """
    kwargs = {}
    json_data = {}
    if key is not None:
        kwargs["key"] = key
    if slug is not None:
        kwargs["slug"] = slug
    if x_immich_checksum is not None:
        kwargs["x_immich_checksum"] = x_immich_checksum
    kwargs["asset_data"] = (asset_data.name, asset_data.read_bytes())
    set_nested(json_data, ["device_asset_id"], device_asset_id)
    set_nested(json_data, ["device_id"], device_id)
    if duration is not None:
        set_nested(json_data, ["duration"], duration)
    set_nested(json_data, ["file_created_at"], file_created_at)
    set_nested(json_data, ["file_modified_at"], file_modified_at)
    if filename is not None:
        set_nested(json_data, ["filename"], filename)
    if is_favorite is not None:
        set_nested(json_data, ["is_favorite"], is_favorite.lower() == "true")
    if live_photo_video_id is not None:
        set_nested(json_data, ["live_photo_video_id"], live_photo_video_id)
    if metadata is not None:
        value_metadata = [json.loads(i) for i in metadata]
        set_nested(json_data, ["metadata"], value_metadata)
    if sidecar_data is not None:
        set_nested(
            json_data, ["sidecar_data"], (sidecar_data.name, sidecar_data.read_bytes())
        )
    if visibility is not None:
        set_nested(json_data, ["visibility"], visibility)
    kwargs.update(json_data)
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.assets, "upload_asset", ctx, **kwargs)
    print_response(result, ctx)


@app.command("view-asset", deprecated=False, rich_help_panel="API commands")
def view_asset(
    ctx: typer.Context,
    id: str = typer.Argument(..., help=""""""),
    edited: Literal["true", "false"] | None = typer.Option(
        None, "--edited", help=""""""
    ),
    key: str | None = typer.Option(None, "--key", help=""""""),
    size: AssetMediaSize | None = typer.Option(None, "--size", help=""""""),
    slug: str | None = typer.Option(None, "--slug", help=""""""),
) -> None:
    """View asset thumbnail

    [link=https://api.immich.app/endpoints/assets/viewAsset]Immich API documentation[/link]
    """
    kwargs = {}
    if edited is not None:
        kwargs["edited"] = edited.lower() == "true"
    kwargs["id"] = id
    if key is not None:
        kwargs["key"] = key
    if size is not None:
        kwargs["size"] = size
    if slug is not None:
        kwargs["slug"] = slug
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.assets, "view_asset", ctx, **kwargs)
    print_response(result, ctx)



"""Generated CLI commands for Stacks tag (auto-generated, do not edit)."""

from __future__ import annotations

import typer
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from immich import AsyncClient

from immich.cli.runtime import print_response, run_command, set_nested
from immich.client.generated.models import *

app = typer.Typer(
    help="""A stack is a group of related assets. One asset is the "primary" asset, and the rest are "child" assets. On the main timeline, stack parents are included by default, while child assets are hidden.\n\n[link=https://api.immich.app/endpoints/stacks]Immich API documentation[/link]"""
)


@app.command("create-stack", deprecated=False, rich_help_panel="API commands")
def create_stack(
    ctx: typer.Context,
    asset_ids: list[str] = typer.Option(
        ..., "--asset-ids", help="""first asset becomes the primary"""
    ),
) -> None:
    """Create a stack

    [link=https://api.immich.app/endpoints/stacks/createStack]Immich API documentation[/link]
    """
    kwargs = {}
    json_data = {}
    set_nested(json_data, ["asset_ids"], asset_ids)
    stack_create_dto = StackCreateDto.model_validate(json_data)
    kwargs["stack_create_dto"] = stack_create_dto
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.stacks, "create_stack", ctx, **kwargs)
    print_response(result, ctx)


@app.command("delete-stack", deprecated=False, rich_help_panel="API commands")
def delete_stack(
    ctx: typer.Context,
    id: str = typer.Argument(..., help=""""""),
) -> None:
    """Delete a stack

    [link=https://api.immich.app/endpoints/stacks/deleteStack]Immich API documentation[/link]
    """
    kwargs = {}
    kwargs["id"] = id
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.stacks, "delete_stack", ctx, **kwargs)
    print_response(result, ctx)


@app.command("delete-stacks", deprecated=False, rich_help_panel="API commands")
def delete_stacks(
    ctx: typer.Context,
    ids: list[str] = typer.Option(..., "--ids", help=""""""),
) -> None:
    """Delete stacks

    [link=https://api.immich.app/endpoints/stacks/deleteStacks]Immich API documentation[/link]
    """
    kwargs = {}
    json_data = {}
    set_nested(json_data, ["ids"], ids)
    bulk_ids_dto = BulkIdsDto.model_validate(json_data)
    kwargs["bulk_ids_dto"] = bulk_ids_dto
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.stacks, "delete_stacks", ctx, **kwargs)
    print_response(result, ctx)


@app.command("get-stack", deprecated=False, rich_help_panel="API commands")
def get_stack(
    ctx: typer.Context,
    id: str = typer.Argument(..., help=""""""),
) -> None:
    """Retrieve a stack

    [link=https://api.immich.app/endpoints/stacks/getStack]Immich API documentation[/link]
    """
    kwargs = {}
    kwargs["id"] = id
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.stacks, "get_stack", ctx, **kwargs)
    print_response(result, ctx)


@app.command(
    "remove-asset-from-stack", deprecated=False, rich_help_panel="API commands"
)
def remove_asset_from_stack(
    ctx: typer.Context,
    asset_id: str = typer.Argument(..., help=""""""),
    id: str = typer.Argument(..., help=""""""),
) -> None:
    """Remove an asset from a stack

    [link=https://api.immich.app/endpoints/stacks/removeAssetFromStack]Immich API documentation[/link]
    """
    kwargs = {}
    kwargs["asset_id"] = asset_id
    kwargs["id"] = id
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(
        client, client.stacks, "remove_asset_from_stack", ctx, **kwargs
    )
    print_response(result, ctx)


@app.command("search-stacks", deprecated=False, rich_help_panel="API commands")
def search_stacks(
    ctx: typer.Context,
    primary_asset_id: str | None = typer.Option(
        None, "--primary-asset-id", help=""""""
    ),
) -> None:
    """Retrieve stacks

    [link=https://api.immich.app/endpoints/stacks/searchStacks]Immich API documentation[/link]
    """
    kwargs = {}
    if primary_asset_id is not None:
        kwargs["primary_asset_id"] = primary_asset_id
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.stacks, "search_stacks", ctx, **kwargs)
    print_response(result, ctx)


@app.command("update-stack", deprecated=False, rich_help_panel="API commands")
def update_stack(
    ctx: typer.Context,
    id: str = typer.Argument(..., help=""""""),
    primary_asset_id: str | None = typer.Option(
        None, "--primary-asset-id", help=""""""
    ),
) -> None:
    """Update a stack

    [link=https://api.immich.app/endpoints/stacks/updateStack]Immich API documentation[/link]
    """
    kwargs = {}
    json_data = {}
    kwargs["id"] = id
    if primary_asset_id is not None:
        set_nested(json_data, ["primary_asset_id"], primary_asset_id)
    stack_update_dto = StackUpdateDto.model_validate(json_data)
    kwargs["stack_update_dto"] = stack_update_dto
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.stacks, "update_stack", ctx, **kwargs)
    print_response(result, ctx)



"""Generated CLI commands for Memories tag (auto-generated, do not edit)."""

from __future__ import annotations

import typer
from datetime import datetime
from typing import Literal, TYPE_CHECKING

if TYPE_CHECKING:
    from immich import AsyncClient

from immich.cli.runtime import print_response, run_command, set_nested
from immich.client.generated.models import *

app = typer.Typer(
    help="""A memory is a specialized collection of assets with dedicated viewing implementations in the web and mobile clients. A memory includes fields related to visibility and are automatically generated per user via a background job.\n\n[link=https://api.immich.app/endpoints/memories]Immich API documentation[/link]"""
)


@app.command("add-memory-assets", deprecated=False, rich_help_panel="API commands")
def add_memory_assets(
    ctx: typer.Context,
    id: str = typer.Argument(..., help=""""""),
    ids: list[str] = typer.Option(..., "--ids", help=""""""),
) -> None:
    """Add assets to a memory

    [link=https://api.immich.app/endpoints/memories/addMemoryAssets]Immich API documentation[/link]
    """
    kwargs = {}
    json_data = {}
    kwargs["id"] = id
    set_nested(json_data, ["ids"], ids)
    bulk_ids_dto = BulkIdsDto.model_validate(json_data)
    kwargs["bulk_ids_dto"] = bulk_ids_dto
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.memories, "add_memory_assets", ctx, **kwargs)
    print_response(result, ctx)


@app.command("create-memory", deprecated=False, rich_help_panel="API commands")
def create_memory(
    ctx: typer.Context,
    asset_ids: list[str] | None = typer.Option(None, "--asset-ids", help=""""""),
    data_year: float = typer.Option(..., "--data-year", help="""""", min=1),
    is_saved: Literal["true", "false"] | None = typer.Option(
        None, "--is-saved", help=""""""
    ),
    memory_at: datetime = typer.Option(..., "--memory-at", help=""""""),
    seen_at: datetime | None = typer.Option(None, "--seen-at", help=""""""),
    type: str = typer.Option(..., "--type", help=""""""),
) -> None:
    """Create a memory

    [link=https://api.immich.app/endpoints/memories/createMemory]Immich API documentation[/link]
    """
    kwargs = {}
    json_data = {}
    if asset_ids is not None:
        set_nested(json_data, ["asset_ids"], asset_ids)
    set_nested(json_data, ["data_year"], data_year)
    if is_saved is not None:
        set_nested(json_data, ["is_saved"], is_saved.lower() == "true")
    set_nested(json_data, ["memory_at"], memory_at)
    if seen_at is not None:
        set_nested(json_data, ["seen_at"], seen_at)
    set_nested(json_data, ["type"], type)
    memory_create_dto = MemoryCreateDto.model_validate(json_data)
    kwargs["memory_create_dto"] = memory_create_dto
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.memories, "create_memory", ctx, **kwargs)
    print_response(result, ctx)


@app.command("delete-memory", deprecated=False, rich_help_panel="API commands")
def delete_memory(
    ctx: typer.Context,
    id: str = typer.Argument(..., help=""""""),
) -> None:
    """Delete a memory

    [link=https://api.immich.app/endpoints/memories/deleteMemory]Immich API documentation[/link]
    """
    kwargs = {}
    kwargs["id"] = id
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.memories, "delete_memory", ctx, **kwargs)
    print_response(result, ctx)


@app.command("get-memory", deprecated=False, rich_help_panel="API commands")
def get_memory(
    ctx: typer.Context,
    id: str = typer.Argument(..., help=""""""),
) -> None:
    """Retrieve a memory

    [link=https://api.immich.app/endpoints/memories/getMemory]Immich API documentation[/link]
    """
    kwargs = {}
    kwargs["id"] = id
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.memories, "get_memory", ctx, **kwargs)
    print_response(result, ctx)


@app.command("memories-statistics", deprecated=False, rich_help_panel="API commands")
def memories_statistics(
    ctx: typer.Context,
    for_: datetime | None = typer.Option(None, "--for", help=""""""),
    is_saved: Literal["true", "false"] | None = typer.Option(
        None, "--is-saved", help=""""""
    ),
    is_trashed: Literal["true", "false"] | None = typer.Option(
        None, "--is-trashed", help=""""""
    ),
    order: MemorySearchOrder | None = typer.Option(None, "--order", help=""""""),
    size: int | None = typer.Option(
        None, "--size", help="""Number of memories to return""", min=1
    ),
    type: MemoryType | None = typer.Option(None, "--type", help=""""""),
) -> None:
    """Retrieve memories statistics

    [link=https://api.immich.app/endpoints/memories/memoriesStatistics]Immich API documentation[/link]
    """
    kwargs = {}
    if for_ is not None:
        kwargs["for_"] = for_
    if is_saved is not None:
        kwargs["is_saved"] = is_saved.lower() == "true"
    if is_trashed is not None:
        kwargs["is_trashed"] = is_trashed.lower() == "true"
    if order is not None:
        kwargs["order"] = order
    if size is not None:
        kwargs["size"] = size
    if type is not None:
        kwargs["type"] = type
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.memories, "memories_statistics", ctx, **kwargs)
    print_response(result, ctx)


@app.command("remove-memory-assets", deprecated=False, rich_help_panel="API commands")
def remove_memory_assets(
    ctx: typer.Context,
    id: str = typer.Argument(..., help=""""""),
    ids: list[str] = typer.Option(..., "--ids", help=""""""),
) -> None:
    """Remove assets from a memory

    [link=https://api.immich.app/endpoints/memories/removeMemoryAssets]Immich API documentation[/link]
    """
    kwargs = {}
    json_data = {}
    kwargs["id"] = id
    set_nested(json_data, ["ids"], ids)
    bulk_ids_dto = BulkIdsDto.model_validate(json_data)
    kwargs["bulk_ids_dto"] = bulk_ids_dto
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.memories, "remove_memory_assets", ctx, **kwargs)
    print_response(result, ctx)


@app.command("search-memories", deprecated=False, rich_help_panel="API commands")
def search_memories(
    ctx: typer.Context,
    for_: datetime | None = typer.Option(None, "--for", help=""""""),
    is_saved: Literal["true", "false"] | None = typer.Option(
        None, "--is-saved", help=""""""
    ),
    is_trashed: Literal["true", "false"] | None = typer.Option(
        None, "--is-trashed", help=""""""
    ),
    order: MemorySearchOrder | None = typer.Option(None, "--order", help=""""""),
    size: int | None = typer.Option(
        None, "--size", help="""Number of memories to return""", min=1
    ),
    type: MemoryType | None = typer.Option(None, "--type", help=""""""),
) -> None:
    """Retrieve memories

    [link=https://api.immich.app/endpoints/memories/searchMemories]Immich API documentation[/link]
    """
    kwargs = {}
    if for_ is not None:
        kwargs["for_"] = for_
    if is_saved is not None:
        kwargs["is_saved"] = is_saved.lower() == "true"
    if is_trashed is not None:
        kwargs["is_trashed"] = is_trashed.lower() == "true"
    if order is not None:
        kwargs["order"] = order
    if size is not None:
        kwargs["size"] = size
    if type is not None:
        kwargs["type"] = type
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.memories, "search_memories", ctx, **kwargs)
    print_response(result, ctx)


@app.command("update-memory", deprecated=False, rich_help_panel="API commands")
def update_memory(
    ctx: typer.Context,
    id: str = typer.Argument(..., help=""""""),
    is_saved: Literal["true", "false"] | None = typer.Option(
        None, "--is-saved", help=""""""
    ),
    memory_at: datetime | None = typer.Option(None, "--memory-at", help=""""""),
    seen_at: datetime | None = typer.Option(None, "--seen-at", help=""""""),
) -> None:
    """Update a memory

    [link=https://api.immich.app/endpoints/memories/updateMemory]Immich API documentation[/link]
    """
    kwargs = {}
    json_data = {}
    kwargs["id"] = id
    if is_saved is not None:
        set_nested(json_data, ["is_saved"], is_saved.lower() == "true")
    if memory_at is not None:
        set_nested(json_data, ["memory_at"], memory_at)
    if seen_at is not None:
        set_nested(json_data, ["seen_at"], seen_at)
    memory_update_dto = MemoryUpdateDto.model_validate(json_data)
    kwargs["memory_update_dto"] = memory_update_dto
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.memories, "update_memory", ctx, **kwargs)
    print_response(result, ctx)



"""Generated CLI commands for System metadata tag (auto-generated, do not edit)."""

from __future__ import annotations

import typer
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from immich import AsyncClient

from immich.cli.runtime import print_response, run_command, set_nested
from immich.client.generated.models import *

app = typer.Typer(
    help="""Endpoints to view, modify, and validate the system metadata, which includes information about things like admin onboarding status.\n\n[link=https://api.immich.app/endpoints/system-metadata]Immich API documentation[/link]"""
)


@app.command("get-admin-onboarding", deprecated=False, rich_help_panel="API commands")
def get_admin_onboarding(
    ctx: typer.Context,
) -> None:
    """Retrieve admin onboarding

    [link=https://api.immich.app/endpoints/system-metadata/getAdminOnboarding]Immich API documentation[/link]
    """
    kwargs = {}
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(
        client, client.system_metadata, "get_admin_onboarding", ctx, **kwargs
    )
    print_response(result, ctx)


@app.command(
    "get-reverse-geocoding-state", deprecated=False, rich_help_panel="API commands"
)
def get_reverse_geocoding_state(
    ctx: typer.Context,
) -> None:
    """Retrieve reverse geocoding state

    [link=https://api.immich.app/endpoints/system-metadata/getReverseGeocodingState]Immich API documentation[/link]
    """
    kwargs = {}
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(
        client, client.system_metadata, "get_reverse_geocoding_state", ctx, **kwargs
    )
    print_response(result, ctx)


@app.command(
    "get-version-check-state", deprecated=False, rich_help_panel="API commands"
)
def get_version_check_state(
    ctx: typer.Context,
) -> None:
    """Retrieve version check state

    [link=https://api.immich.app/endpoints/system-metadata/getVersionCheckState]Immich API documentation[/link]
    """
    kwargs = {}
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(
        client, client.system_metadata, "get_version_check_state", ctx, **kwargs
    )
    print_response(result, ctx)


@app.command(
    "update-admin-onboarding", deprecated=False, rich_help_panel="API commands"
)
def update_admin_onboarding(
    ctx: typer.Context,
    is_onboarded: bool = typer.Option(..., "--is-onboarded", help=""""""),
) -> None:
    """Update admin onboarding

    [link=https://api.immich.app/endpoints/system-metadata/updateAdminOnboarding]Immich API documentation[/link]
    """
    kwargs = {}
    json_data = {}
    set_nested(json_data, ["is_onboarded"], is_onboarded)
    admin_onboarding_update_dto = AdminOnboardingUpdateDto.model_validate(json_data)
    kwargs["admin_onboarding_update_dto"] = admin_onboarding_update_dto
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(
        client, client.system_metadata, "update_admin_onboarding", ctx, **kwargs
    )
    print_response(result, ctx)



"""Generated CLI commands for Workflows tag (auto-generated, do not edit)."""

from __future__ import annotations

import typer
import json
from typing import Literal, TYPE_CHECKING

if TYPE_CHECKING:
    from immich import AsyncClient

from immich.cli.runtime import print_response, run_command, set_nested
from immich.client.generated.models import *

app = typer.Typer(
    help="""A workflow is a set of actions that run whenever a triggering event occurs. Workflows also can include filters to further limit execution.\n\n[link=https://api.immich.app/endpoints/workflows]Immich API documentation[/link]"""
)


@app.command("create-workflow", deprecated=False, rich_help_panel="API commands")
def create_workflow(
    ctx: typer.Context,
    actions: list[str] = typer.Option(..., "--actions", help="""As a JSON string"""),
    description: str | None = typer.Option(None, "--description", help=""""""),
    enabled: Literal["true", "false"] | None = typer.Option(
        None, "--enabled", help=""""""
    ),
    filters: list[str] = typer.Option(..., "--filters", help="""As a JSON string"""),
    name: str = typer.Option(..., "--name", help=""""""),
    trigger_type: str = typer.Option(..., "--trigger-type", help=""""""),
) -> None:
    """Create a workflow

    [link=https://api.immich.app/endpoints/workflows/createWorkflow]Immich API documentation[/link]
    """
    kwargs = {}
    json_data = {}
    value_actions = [json.loads(i) for i in actions]
    set_nested(json_data, ["actions"], value_actions)
    if description is not None:
        set_nested(json_data, ["description"], description)
    if enabled is not None:
        set_nested(json_data, ["enabled"], enabled.lower() == "true")
    value_filters = [json.loads(i) for i in filters]
    set_nested(json_data, ["filters"], value_filters)
    set_nested(json_data, ["name"], name)
    set_nested(json_data, ["trigger_type"], trigger_type)
    workflow_create_dto = WorkflowCreateDto.model_validate(json_data)
    kwargs["workflow_create_dto"] = workflow_create_dto
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.workflows, "create_workflow", ctx, **kwargs)
    print_response(result, ctx)


@app.command("delete-workflow", deprecated=False, rich_help_panel="API commands")
def delete_workflow(
    ctx: typer.Context,
    id: str = typer.Argument(..., help=""""""),
) -> None:
    """Delete a workflow

    [link=https://api.immich.app/endpoints/workflows/deleteWorkflow]Immich API documentation[/link]
    """
    kwargs = {}
    kwargs["id"] = id
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.workflows, "delete_workflow", ctx, **kwargs)
    print_response(result, ctx)


@app.command("get-workflow", deprecated=False, rich_help_panel="API commands")
def get_workflow(
    ctx: typer.Context,
    id: str = typer.Argument(..., help=""""""),
) -> None:
    """Retrieve a workflow

    [link=https://api.immich.app/endpoints/workflows/getWorkflow]Immich API documentation[/link]
    """
    kwargs = {}
    kwargs["id"] = id
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.workflows, "get_workflow", ctx, **kwargs)
    print_response(result, ctx)


@app.command("get-workflows", deprecated=False, rich_help_panel="API commands")
def get_workflows(
    ctx: typer.Context,
) -> None:
    """List all workflows

    [link=https://api.immich.app/endpoints/workflows/getWorkflows]Immich API documentation[/link]
    """
    kwargs = {}
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.workflows, "get_workflows", ctx, **kwargs)
    print_response(result, ctx)


@app.command("update-workflow", deprecated=False, rich_help_panel="API commands")
def update_workflow(
    ctx: typer.Context,
    id: str = typer.Argument(..., help=""""""),
    actions: list[str] | None = typer.Option(
        None, "--actions", help="""As a JSON string"""
    ),
    description: str | None = typer.Option(None, "--description", help=""""""),
    enabled: Literal["true", "false"] | None = typer.Option(
        None, "--enabled", help=""""""
    ),
    filters: list[str] | None = typer.Option(
        None, "--filters", help="""As a JSON string"""
    ),
    name: str | None = typer.Option(None, "--name", help=""""""),
    trigger_type: str | None = typer.Option(None, "--trigger-type", help=""""""),
) -> None:
    """Update a workflow

    [link=https://api.immich.app/endpoints/workflows/updateWorkflow]Immich API documentation[/link]
    """
    kwargs = {}
    json_data = {}
    kwargs["id"] = id
    if actions is not None:
        value_actions = [json.loads(i) for i in actions]
        set_nested(json_data, ["actions"], value_actions)
    if description is not None:
        set_nested(json_data, ["description"], description)
    if enabled is not None:
        set_nested(json_data, ["enabled"], enabled.lower() == "true")
    if filters is not None:
        value_filters = [json.loads(i) for i in filters]
        set_nested(json_data, ["filters"], value_filters)
    if name is not None:
        set_nested(json_data, ["name"], name)
    if trigger_type is not None:
        set_nested(json_data, ["trigger_type"], trigger_type)
    workflow_update_dto = WorkflowUpdateDto.model_validate(json_data)
    kwargs["workflow_update_dto"] = workflow_update_dto
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.workflows, "update_workflow", ctx, **kwargs)
    print_response(result, ctx)



"""Generated CLI commands for Server tag (auto-generated, do not edit)."""

from __future__ import annotations

import typer
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from immich import AsyncClient

from immich.cli.runtime import print_response, run_command, set_nested
from immich.client.generated.models import *

app = typer.Typer(
    help="""Information about the current server deployment, including version and build information, available features, supported media types, and more.\n\n[link=https://api.immich.app/endpoints/server]Immich API documentation[/link]"""
)


@app.command("delete-server-license", deprecated=False, rich_help_panel="API commands")
def delete_server_license(
    ctx: typer.Context,
) -> None:
    """Delete server product key

    [link=https://api.immich.app/endpoints/server/deleteServerLicense]Immich API documentation[/link]
    """
    kwargs = {}
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.server, "delete_server_license", ctx, **kwargs)
    print_response(result, ctx)


@app.command("get-about-info", deprecated=False, rich_help_panel="API commands")
def get_about_info(
    ctx: typer.Context,
) -> None:
    """Get server information

    [link=https://api.immich.app/endpoints/server/getAboutInfo]Immich API documentation[/link]
    """
    kwargs = {}
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.server, "get_about_info", ctx, **kwargs)
    print_response(result, ctx)


@app.command("get-apk-links", deprecated=False, rich_help_panel="API commands")
def get_apk_links(
    ctx: typer.Context,
) -> None:
    """Get APK links

    [link=https://api.immich.app/endpoints/server/getApkLinks]Immich API documentation[/link]
    """
    kwargs = {}
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.server, "get_apk_links", ctx, **kwargs)
    print_response(result, ctx)


@app.command("get-server-config", deprecated=False, rich_help_panel="API commands")
def get_server_config(
    ctx: typer.Context,
) -> None:
    """Get config

    [link=https://api.immich.app/endpoints/server/getServerConfig]Immich API documentation[/link]
    """
    kwargs = {}
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.server, "get_server_config", ctx, **kwargs)
    print_response(result, ctx)


@app.command("get-server-features", deprecated=False, rich_help_panel="API commands")
def get_server_features(
    ctx: typer.Context,
) -> None:
    """Get features

    [link=https://api.immich.app/endpoints/server/getServerFeatures]Immich API documentation[/link]
    """
    kwargs = {}
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.server, "get_server_features", ctx, **kwargs)
    print_response(result, ctx)


@app.command("get-server-license", deprecated=False, rich_help_panel="API commands")
def get_server_license(
    ctx: typer.Context,
) -> None:
    """Get product key

    [link=https://api.immich.app/endpoints/server/getServerLicense]Immich API documentation[/link]
    """
    kwargs = {}
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.server, "get_server_license", ctx, **kwargs)
    print_response(result, ctx)


@app.command("get-server-statistics", deprecated=False, rich_help_panel="API commands")
def get_server_statistics(
    ctx: typer.Context,
) -> None:
    """Get statistics

    [link=https://api.immich.app/endpoints/server/getServerStatistics]Immich API documentation[/link]
    """
    kwargs = {}
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.server, "get_server_statistics", ctx, **kwargs)
    print_response(result, ctx)


@app.command("get-server-version", deprecated=False, rich_help_panel="API commands")
def get_server_version(
    ctx: typer.Context,
) -> None:
    """Get server version

    [link=https://api.immich.app/endpoints/server/getServerVersion]Immich API documentation[/link]
    """
    kwargs = {}
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.server, "get_server_version", ctx, **kwargs)
    print_response(result, ctx)


@app.command("get-storage", deprecated=False, rich_help_panel="API commands")
def get_storage(
    ctx: typer.Context,
) -> None:
    """Get storage

    [link=https://api.immich.app/endpoints/server/getStorage]Immich API documentation[/link]
    """
    kwargs = {}
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.server, "get_storage", ctx, **kwargs)
    print_response(result, ctx)


@app.command(
    "get-supported-media-types", deprecated=False, rich_help_panel="API commands"
)
def get_supported_media_types(
    ctx: typer.Context,
) -> None:
    """Get supported media types

    [link=https://api.immich.app/endpoints/server/getSupportedMediaTypes]Immich API documentation[/link]
    """
    kwargs = {}
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(
        client, client.server, "get_supported_media_types", ctx, **kwargs
    )
    print_response(result, ctx)


@app.command("get-theme", deprecated=False, rich_help_panel="API commands")
def get_theme(
    ctx: typer.Context,
) -> None:
    """Get theme

    [link=https://api.immich.app/endpoints/server/getTheme]Immich API documentation[/link]
    """
    kwargs = {}
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.server, "get_theme", ctx, **kwargs)
    print_response(result, ctx)


@app.command("get-version-check", deprecated=False, rich_help_panel="API commands")
def get_version_check(
    ctx: typer.Context,
) -> None:
    """Get version check status

    [link=https://api.immich.app/endpoints/server/getVersionCheck]Immich API documentation[/link]
    """
    kwargs = {}
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.server, "get_version_check", ctx, **kwargs)
    print_response(result, ctx)


@app.command("get-version-history", deprecated=False, rich_help_panel="API commands")
def get_version_history(
    ctx: typer.Context,
) -> None:
    """Get version history

    [link=https://api.immich.app/endpoints/server/getVersionHistory]Immich API documentation[/link]
    """
    kwargs = {}
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.server, "get_version_history", ctx, **kwargs)
    print_response(result, ctx)


@app.command("ping-server", deprecated=False, rich_help_panel="API commands")
def ping_server(
    ctx: typer.Context,
) -> None:
    """Ping

    [link=https://api.immich.app/endpoints/server/pingServer]Immich API documentation[/link]
    """
    kwargs = {}
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.server, "ping_server", ctx, **kwargs)
    print_response(result, ctx)


@app.command("set-server-license", deprecated=False, rich_help_panel="API commands")
def set_server_license(
    ctx: typer.Context,
    activation_key: str = typer.Option(..., "--activation-key", help=""""""),
    license_key: str = typer.Option(..., "--license-key", help=""""""),
) -> None:
    """Set server product key

    [link=https://api.immich.app/endpoints/server/setServerLicense]Immich API documentation[/link]
    """
    kwargs = {}
    json_data = {}
    set_nested(json_data, ["activation_key"], activation_key)
    set_nested(json_data, ["license_key"], license_key)
    license_key_dto = LicenseKeyDto.model_validate(json_data)
    kwargs["license_key_dto"] = license_key_dto
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.server, "set_server_license", ctx, **kwargs)
    print_response(result, ctx)



"""Generated CLI commands for Partners tag (auto-generated, do not edit)."""

from __future__ import annotations

import typer
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from immich import AsyncClient

from immich.cli.runtime import print_response, run_command, set_nested
from immich.client.generated.models import *

app = typer.Typer(
    help="""A partner is a link with another user that allows sharing of assets between two users.\n\n[link=https://api.immich.app/endpoints/partners]Immich API documentation[/link]"""
)


@app.command("create-partner", deprecated=False, rich_help_panel="API commands")
def create_partner(
    ctx: typer.Context,
    shared_with_id: str = typer.Option(..., "--shared-with-id", help=""""""),
) -> None:
    """Create a partner

    [link=https://api.immich.app/endpoints/partners/createPartner]Immich API documentation[/link]
    """
    kwargs = {}
    json_data = {}
    set_nested(json_data, ["shared_with_id"], shared_with_id)
    partner_create_dto = PartnerCreateDto.model_validate(json_data)
    kwargs["partner_create_dto"] = partner_create_dto
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.partners, "create_partner", ctx, **kwargs)
    print_response(result, ctx)


@app.command(
    "create-partner-deprecated", deprecated=True, rich_help_panel="API commands"
)
def create_partner_deprecated(
    ctx: typer.Context,
    id: str = typer.Argument(..., help=""""""),
) -> None:
    """Create a partner

    [link=https://api.immich.app/endpoints/partners/createPartnerDeprecated]Immich API documentation[/link]
    """
    kwargs = {}
    kwargs["id"] = id
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(
        client, client.partners, "create_partner_deprecated", ctx, **kwargs
    )
    print_response(result, ctx)


@app.command("get-partners", deprecated=False, rich_help_panel="API commands")
def get_partners(
    ctx: typer.Context,
    direction: PartnerDirection = typer.Option(..., "--direction", help=""""""),
) -> None:
    """Retrieve partners

    [link=https://api.immich.app/endpoints/partners/getPartners]Immich API documentation[/link]
    """
    kwargs = {}
    kwargs["direction"] = direction
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.partners, "get_partners", ctx, **kwargs)
    print_response(result, ctx)


@app.command("remove-partner", deprecated=False, rich_help_panel="API commands")
def remove_partner(
    ctx: typer.Context,
    id: str = typer.Argument(..., help=""""""),
) -> None:
    """Remove a partner

    [link=https://api.immich.app/endpoints/partners/removePartner]Immich API documentation[/link]
    """
    kwargs = {}
    kwargs["id"] = id
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.partners, "remove_partner", ctx, **kwargs)
    print_response(result, ctx)


@app.command("update-partner", deprecated=False, rich_help_panel="API commands")
def update_partner(
    ctx: typer.Context,
    id: str = typer.Argument(..., help=""""""),
    in_timeline: bool = typer.Option(..., "--in-timeline", help=""""""),
) -> None:
    """Update a partner

    [link=https://api.immich.app/endpoints/partners/updatePartner]Immich API documentation[/link]
    """
    kwargs = {}
    json_data = {}
    kwargs["id"] = id
    set_nested(json_data, ["in_timeline"], in_timeline)
    partner_update_dto = PartnerUpdateDto.model_validate(json_data)
    kwargs["partner_update_dto"] = partner_update_dto
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.partners, "update_partner", ctx, **kwargs)
    print_response(result, ctx)



try:  # pragma: no cover
    from .main import app
except ImportError:  # pragma: no cover
    import sys

    message = 'To use the immich command, install the "cli" extra:\n\n\t- pip install immich[cli]\n\tOR\n\t- uv add immich --extra cli\n'
    print(message, file=sys.stderr)
    sys.exit(1)


def main() -> None:  # pragma: no cover
    app()



"""Generated CLI commands for Trash tag (auto-generated, do not edit)."""

from __future__ import annotations

import typer
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from immich import AsyncClient

from immich.cli.runtime import print_response, run_command, set_nested
from immich.client.generated.models import *

app = typer.Typer(
    help="""Endpoints for managing the trash can, which includes assets that have been discarded. Items in the trash are automatically deleted after a configured amount of time.\n\n[link=https://api.immich.app/endpoints/trash]Immich API documentation[/link]"""
)


@app.command("empty-trash", deprecated=False, rich_help_panel="API commands")
def empty_trash(
    ctx: typer.Context,
) -> None:
    """Empty trash

    [link=https://api.immich.app/endpoints/trash/emptyTrash]Immich API documentation[/link]
    """
    kwargs = {}
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.trash, "empty_trash", ctx, **kwargs)
    print_response(result, ctx)


@app.command("restore-assets", deprecated=False, rich_help_panel="API commands")
def restore_assets(
    ctx: typer.Context,
    ids: list[str] = typer.Option(..., "--ids", help=""""""),
) -> None:
    """Restore assets

    [link=https://api.immich.app/endpoints/trash/restoreAssets]Immich API documentation[/link]
    """
    kwargs = {}
    json_data = {}
    set_nested(json_data, ["ids"], ids)
    bulk_ids_dto = BulkIdsDto.model_validate(json_data)
    kwargs["bulk_ids_dto"] = bulk_ids_dto
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.trash, "restore_assets", ctx, **kwargs)
    print_response(result, ctx)


@app.command("restore-trash", deprecated=False, rich_help_panel="API commands")
def restore_trash(
    ctx: typer.Context,
) -> None:
    """Restore trash

    [link=https://api.immich.app/endpoints/trash/restoreTrash]Immich API documentation[/link]
    """
    kwargs = {}
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.trash, "restore_trash", ctx, **kwargs)
    print_response(result, ctx)



from __future__ import annotations

from pathlib import Path
from typing import Any, Optional
from uuid import UUID

from pydantic import StrictStr

from immich.client.generated.api.albums_api import AlbumsApi
from immich.client.generated.api.assets_api import AssetsApi
from immich.client.generated.api.server_api import ServerApi
from immich.client.generated.models.asset_media_size import AssetMediaSize
from immich.client.utils.upload import (
    check_duplicates as check_dupes,
    delete_files,
    scan_files,
    update_albums,
    upload_files,
)
from immich.client.utils.download import download_file, resolve_output_filename
from immich.client.types import HeadersType, UploadResult, UploadStats


class AssetsApiWrapped(AssetsApi):
    """Wrapper for the AssetsApi that provides convenience methods."""

    async def download_asset_to_file(
        self,
        id: UUID,
        out_dir: Path,
        key: Optional[StrictStr] = None,
        slug: Optional[StrictStr] = None,
        filename: Optional[str] = None,
        show_progress: bool = False,
        **kwargs: Any,
    ) -> Path:
        """
        Download an asset to a file.

        :param id: The asset ID.
        :param out_dir: The directory to write the asset to.
        :param key: Public share key (the last path segment of a public share URL, i.e. `/share/<key>`). When provided, the asset can be accessed via the public share link without an API key. Typically you pass either `key` or `slug`.
        :param slug: Public share slug for custom share URLs (the last path segment of `/s/<slug>`). Allows access without authentication. Typically you pass either `slug` or `key`.
        :param filename: The filename to use. If not provided, we use the original filename from the headers or default to "orig-" + asset_id.
        :param show_progress: Whether to show a progress bar while downloading.
        :param kwargs: Additional arguments to pass to the `download_asset_without_preload_content` method.
        :return: The path to the downloaded file.

        For exact request/response behavior, inspect `AssetsApi.download_asset_without_preload_content`
        in the generated client.
        """
        out_dir.mkdir(parents=True, exist_ok=True)

        def make_request(extra_headers: Optional[HeadersType]):
            return self.download_asset_without_preload_content(
                id=id,
                key=key,
                slug=slug,
                _headers=kwargs.get("_headers", {}) | (extra_headers or {}),
                **kwargs,
            )

        return await download_file(
            make_request=make_request,
            out_dir=out_dir,
            resolve_filename=lambda headers: resolve_output_filename(
                headers,
                name=filename,
                default_base=f"orig-{id}",
            ),
            show_progress=show_progress,
        )

    async def play_asset_video_to_file(
        self,
        id: UUID,
        out_dir: Path,
        key: Optional[StrictStr] = None,
        slug: Optional[StrictStr] = None,
        filename: Optional[str] = None,
        show_progress: bool = False,
        **kwargs: Any,
    ) -> Path:
        """
        Save an asset's video stream to a file.

        :param id: The asset ID.
        :param out_dir: The directory to write the video to.
        :param key: Public share key (the last path segment of a public share URL, i.e. `/share/<key>`). When provided, the asset can be accessed via the public share link without an API key. Typically you pass either `key` or `slug`.
        :param slug: Public share slug for custom share URLs (the last path segment of `/s/<slug>`). Allows access without authentication. Typically you pass either `slug` or `key`.
        :param filename: The filename to use. If not provided, we use the original filename from the headers or default to "video-" + asset_id.
        :param show_progress: Whether to show a progress bar while downloading.
        :param kwargs: Additional arguments to pass to the [AssetsApi.play_asset_video_without_preload_content][] method.
        :return: The path to the downloaded file.
        """
        out_dir.mkdir(parents=True, exist_ok=True)

        def make_request(extra_headers: Optional[HeadersType]):
            return self.play_asset_video_without_preload_content(
                id=id,
                key=key,
                slug=slug,
                _headers=kwargs.get("_headers", {}) | (extra_headers or {}),
                **kwargs,
            )

        return await download_file(
            make_request=make_request,
            out_dir=out_dir,
            resolve_filename=lambda headers: resolve_output_filename(
                headers,
                name=filename,
                default_base=f"video-{id}",
            ),
            show_progress=show_progress,
        )

    async def view_asset_to_file(
        self,
        id: UUID,
        out_dir: Path,
        key: Optional[StrictStr] = None,
        size: Optional[AssetMediaSize] = None,
        slug: Optional[StrictStr] = None,
        filename: Optional[str] = None,
        show_progress: bool = False,
        **kwargs: Any,
    ) -> Path:
        """
        Save an asset's thumbnail to a file.

        :param id: The asset ID.
        :param out_dir: The directory to write the asset to.
        :param key: Public share key (the last path segment of a public share URL, i.e. `/share/<key>`). When provided, the thumbnail can be fetched via the public share link without an API key. Typically you pass either `key` or `slug`.
        :param size: Thumbnail size.
        :param slug: Public share slug for custom share URLs (the last path segment of `/s/<slug>`). Allows access without authentication. Typically you pass either `slug` or `key`.
        :param filename: The filename to use. If not provided, we use the original filename from the headers or default to "thumb-" + asset_id.
        :param show_progress: Whether to show a progress bar while downloading.
        :param kwargs: Additional arguments to pass to the [AssetsApi.view_asset_without_preload_content][] method.
        :return: The path to the downloaded file.
        """
        out_dir.mkdir(parents=True, exist_ok=True)

        def make_request(extra_headers: Optional[HeadersType]):
            return self.view_asset_without_preload_content(
                id=id,
                key=key,
                size=size,
                slug=slug,
                _headers=kwargs.get("_headers", {}) | (extra_headers or {}),
                **kwargs,
            )

        return await download_file(
            make_request=make_request,
            out_dir=out_dir,
            resolve_filename=lambda headers: resolve_output_filename(
                headers,
                name=filename,
                default_base=f"thumb-{id}",
            ),
            show_progress=show_progress,
        )

    async def upload(
        self,
        paths: Path | list[Path] | str | list[str],
        *,
        ignore_pattern: Optional[str] = None,
        include_hidden: bool = False,
        skip_duplicates: bool = False,
        concurrency: int = 5,
        show_progress: bool = False,
        album_name: Optional[str] = None,
        delete_uploads: bool = False,
        delete_duplicates: bool = False,
        dry_run: bool = False,
    ) -> UploadResult:
        """
        Upload assets with smart features (duplicate detection, album management, sidecar support, dry run).

        :param paths: File or directory paths to upload. Can be a single path or list of paths. Directories are automatically walked recursively. To ignore subdirectories, use the `ignore_pattern` parameter.
        :param ignore_pattern: Wildcard pattern to ignore files (uses `fnmatch` stdlib module, not regex). Examples: "*.tmp" (ignore all .tmp files), "*/subdir/*" (ignore files in subdir at any level).
        :param include_hidden: Whether to include hidden files (starting with ".").
        :param skip_duplicates: Whether to skip duplicate checking (might still get rejected on the server).
        :param concurrency: Number of concurrent uploads. Defaults to 5. A higher number may increase upload speed, but also increases the risk of rate limiting or other issues.
        :param show_progress: Whether to show progress bars.
        :param album_name: Album name to create or use. If None, no album operations are performed.
        :param delete_uploads: Whether to delete successfully uploaded files locally.
        :param delete_duplicates: Whether to delete duplicate files locally.
        :param dry_run: Simulate uploads without actually uploading.
        :return: UploadResult with uploaded assets, rejected files, failures, and statistics.
        """
        if concurrency < 1:
            raise ValueError("concurrency must be >= 1")
        server_api = ServerApi(self.api_client)
        albums_api = AlbumsApi(self.api_client)

        _paths = [paths] if isinstance(paths, (str, Path)) else paths
        _paths = [Path(p) for p in _paths]

        files = await scan_files(_paths, server_api, ignore_pattern, include_hidden)
        if not files:
            return UploadResult(
                uploaded=[],
                rejected=[],
                failed=[],
                stats=UploadStats(total=0, uploaded=0, rejected=0, failed=0),
            )

        new_files, checked_rejected = await check_dupes(
            files=files,
            assets_api=self,
            skip_duplicates=skip_duplicates,
            show_progress=show_progress,
            dry_run=dry_run,
        )

        uploaded, actual_rejected, failed = await upload_files(
            files=new_files,
            assets_api=self,
            concurrency=concurrency,
            show_progress=show_progress,
            dry_run=dry_run,
        )

        if album_name and not dry_run:
            await update_albums(
                uploaded=uploaded, album_name=album_name, albums_api=albums_api
            )

        # we can either check pre-upload rejected files or on-upload rejected files, so we return the appropriate list
        # alternative would be to use both lists and deduplicate by asset_id, however adds overhead and assumes the API returned different results
        rejected = actual_rejected if skip_duplicates else checked_rejected

        await delete_files(
            uploaded=uploaded,
            rejected=rejected,
            delete_uploads=delete_uploads,
            delete_duplicates=delete_duplicates,
            dry_run=dry_run,
        )

        return UploadResult(
            uploaded=uploaded,
            rejected=rejected,
            failed=failed,
            stats=UploadStats(
                total=len(files),
                uploaded=len(uploaded),
                rejected=len(rejected),
                failed=len(failed),
            ),
        )



"""Generated CLI commands for Users tag (auto-generated, do not edit)."""

from __future__ import annotations

import typer
from pathlib import Path
from typing import Literal, TYPE_CHECKING

if TYPE_CHECKING:
    from immich import AsyncClient

from immich.cli.runtime import print_response, run_command, set_nested
from immich.client.generated.models import *

app = typer.Typer(
    help="""Endpoints for viewing and updating the current users, including product key information, profile picture data, onboarding progress, and more.\n\n[link=https://api.immich.app/endpoints/users]Immich API documentation[/link]"""
)


@app.command("create-profile-image", deprecated=False, rich_help_panel="API commands")
def create_profile_image(
    ctx: typer.Context,
    file: Path = typer.Option(..., "--file", help="""""", exists=True),
) -> None:
    """Create user profile image

    [link=https://api.immich.app/endpoints/users/createProfileImage]Immich API documentation[/link]
    """
    kwargs = {}
    json_data = {}
    kwargs["file"] = (file.name, file.read_bytes())
    kwargs.update(json_data)
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.users, "create_profile_image", ctx, **kwargs)
    print_response(result, ctx)


@app.command("delete-profile-image", deprecated=False, rich_help_panel="API commands")
def delete_profile_image(
    ctx: typer.Context,
) -> None:
    """Delete user profile image

    [link=https://api.immich.app/endpoints/users/deleteProfileImage]Immich API documentation[/link]
    """
    kwargs = {}
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.users, "delete_profile_image", ctx, **kwargs)
    print_response(result, ctx)


@app.command("delete-user-license", deprecated=False, rich_help_panel="API commands")
def delete_user_license(
    ctx: typer.Context,
) -> None:
    """Delete user product key

    [link=https://api.immich.app/endpoints/users/deleteUserLicense]Immich API documentation[/link]
    """
    kwargs = {}
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.users, "delete_user_license", ctx, **kwargs)
    print_response(result, ctx)


@app.command("delete-user-onboarding", deprecated=False, rich_help_panel="API commands")
def delete_user_onboarding(
    ctx: typer.Context,
) -> None:
    """Delete user onboarding

    [link=https://api.immich.app/endpoints/users/deleteUserOnboarding]Immich API documentation[/link]
    """
    kwargs = {}
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.users, "delete_user_onboarding", ctx, **kwargs)
    print_response(result, ctx)


@app.command("get-my-preferences", deprecated=False, rich_help_panel="API commands")
def get_my_preferences(
    ctx: typer.Context,
) -> None:
    """Get my preferences

    [link=https://api.immich.app/endpoints/users/getMyPreferences]Immich API documentation[/link]
    """
    kwargs = {}
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.users, "get_my_preferences", ctx, **kwargs)
    print_response(result, ctx)


@app.command("get-my-user", deprecated=False, rich_help_panel="API commands")
def get_my_user(
    ctx: typer.Context,
) -> None:
    """Get current user

    [link=https://api.immich.app/endpoints/users/getMyUser]Immich API documentation[/link]
    """
    kwargs = {}
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.users, "get_my_user", ctx, **kwargs)
    print_response(result, ctx)


@app.command("get-profile-image", deprecated=False, rich_help_panel="API commands")
def get_profile_image(
    ctx: typer.Context,
    id: str = typer.Argument(..., help=""""""),
) -> None:
    """Retrieve user profile image

    [link=https://api.immich.app/endpoints/users/getProfileImage]Immich API documentation[/link]
    """
    kwargs = {}
    kwargs["id"] = id
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.users, "get_profile_image", ctx, **kwargs)
    print_response(result, ctx)


@app.command("get-user", deprecated=False, rich_help_panel="API commands")
def get_user(
    ctx: typer.Context,
    id: str = typer.Argument(..., help=""""""),
) -> None:
    """Retrieve a user

    [link=https://api.immich.app/endpoints/users/getUser]Immich API documentation[/link]
    """
    kwargs = {}
    kwargs["id"] = id
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.users, "get_user", ctx, **kwargs)
    print_response(result, ctx)


@app.command("get-user-license", deprecated=False, rich_help_panel="API commands")
def get_user_license(
    ctx: typer.Context,
) -> None:
    """Retrieve user product key

    [link=https://api.immich.app/endpoints/users/getUserLicense]Immich API documentation[/link]
    """
    kwargs = {}
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.users, "get_user_license", ctx, **kwargs)
    print_response(result, ctx)


@app.command("get-user-onboarding", deprecated=False, rich_help_panel="API commands")
def get_user_onboarding(
    ctx: typer.Context,
) -> None:
    """Retrieve user onboarding

    [link=https://api.immich.app/endpoints/users/getUserOnboarding]Immich API documentation[/link]
    """
    kwargs = {}
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.users, "get_user_onboarding", ctx, **kwargs)
    print_response(result, ctx)


@app.command("search-users", deprecated=False, rich_help_panel="API commands")
def search_users(
    ctx: typer.Context,
) -> None:
    """Get all users

    [link=https://api.immich.app/endpoints/users/searchUsers]Immich API documentation[/link]
    """
    kwargs = {}
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.users, "search_users", ctx, **kwargs)
    print_response(result, ctx)


@app.command("set-user-license", deprecated=False, rich_help_panel="API commands")
def set_user_license(
    ctx: typer.Context,
    activation_key: str = typer.Option(..., "--activation-key", help=""""""),
    license_key: str = typer.Option(..., "--license-key", help=""""""),
) -> None:
    """Set user product key

    [link=https://api.immich.app/endpoints/users/setUserLicense]Immich API documentation[/link]
    """
    kwargs = {}
    json_data = {}
    set_nested(json_data, ["activation_key"], activation_key)
    set_nested(json_data, ["license_key"], license_key)
    license_key_dto = LicenseKeyDto.model_validate(json_data)
    kwargs["license_key_dto"] = license_key_dto
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.users, "set_user_license", ctx, **kwargs)
    print_response(result, ctx)


@app.command("set-user-onboarding", deprecated=False, rich_help_panel="API commands")
def set_user_onboarding(
    ctx: typer.Context,
    is_onboarded: bool = typer.Option(..., "--is-onboarded", help=""""""),
) -> None:
    """Update user onboarding

    [link=https://api.immich.app/endpoints/users/setUserOnboarding]Immich API documentation[/link]
    """
    kwargs = {}
    json_data = {}
    set_nested(json_data, ["is_onboarded"], is_onboarded)
    onboarding_dto = OnboardingDto.model_validate(json_data)
    kwargs["onboarding_dto"] = onboarding_dto
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.users, "set_user_onboarding", ctx, **kwargs)
    print_response(result, ctx)


@app.command("update-my-preferences", deprecated=False, rich_help_panel="API commands")
def update_my_preferences(
    ctx: typer.Context,
    albums_default_asset_order: str | None = typer.Option(
        None, "--albums-default-asset-order", help=""""""
    ),
    avatar_color: str | None = typer.Option(None, "--avatar-color", help=""""""),
    cast_g_cast_enabled: Literal["true", "false"] | None = typer.Option(
        None, "--cast-g-cast-enabled", help=""""""
    ),
    download_archive_size: int | None = typer.Option(
        None, "--download-archive-size", help="""""", min=1
    ),
    download_include_embedded_videos: Literal["true", "false"] | None = typer.Option(
        None, "--download-include-embedded-videos", help=""""""
    ),
    email_notifications_album_invite: Literal["true", "false"] | None = typer.Option(
        None, "--email-notifications-album-invite", help=""""""
    ),
    email_notifications_album_update: Literal["true", "false"] | None = typer.Option(
        None, "--email-notifications-album-update", help=""""""
    ),
    email_notifications_enabled: Literal["true", "false"] | None = typer.Option(
        None, "--email-notifications-enabled", help=""""""
    ),
    folders_enabled: Literal["true", "false"] | None = typer.Option(
        None, "--folders-enabled", help=""""""
    ),
    folders_sidebar_web: Literal["true", "false"] | None = typer.Option(
        None, "--folders-sidebar-web", help=""""""
    ),
    memories_duration: int | None = typer.Option(
        None, "--memories-duration", help="""""", min=1
    ),
    memories_enabled: Literal["true", "false"] | None = typer.Option(
        None, "--memories-enabled", help=""""""
    ),
    people_enabled: Literal["true", "false"] | None = typer.Option(
        None, "--people-enabled", help=""""""
    ),
    people_sidebar_web: Literal["true", "false"] | None = typer.Option(
        None, "--people-sidebar-web", help=""""""
    ),
    purchase_hide_buy_button_until: str | None = typer.Option(
        None, "--purchase-hide-buy-button-until", help=""""""
    ),
    purchase_show_support_badge: Literal["true", "false"] | None = typer.Option(
        None, "--purchase-show-support-badge", help=""""""
    ),
    ratings_enabled: Literal["true", "false"] | None = typer.Option(
        None, "--ratings-enabled", help=""""""
    ),
    shared_links_enabled: Literal["true", "false"] | None = typer.Option(
        None, "--shared-links-enabled", help=""""""
    ),
    shared_links_sidebar_web: Literal["true", "false"] | None = typer.Option(
        None, "--shared-links-sidebar-web", help=""""""
    ),
    tags_enabled: Literal["true", "false"] | None = typer.Option(
        None, "--tags-enabled", help=""""""
    ),
    tags_sidebar_web: Literal["true", "false"] | None = typer.Option(
        None, "--tags-sidebar-web", help=""""""
    ),
) -> None:
    """Update my preferences

    [link=https://api.immich.app/endpoints/users/updateMyPreferences]Immich API documentation[/link]
    """
    kwargs = {}
    json_data = {}
    if albums_default_asset_order is not None:
        set_nested(
            json_data, ["albums_default_asset_order"], albums_default_asset_order
        )
    if avatar_color is not None:
        set_nested(json_data, ["avatar_color"], avatar_color)
    if cast_g_cast_enabled is not None:
        set_nested(
            json_data, ["cast_g_cast_enabled"], cast_g_cast_enabled.lower() == "true"
        )
    if download_archive_size is not None:
        set_nested(json_data, ["download_archive_size"], download_archive_size)
    if download_include_embedded_videos is not None:
        set_nested(
            json_data,
            ["download_include_embedded_videos"],
            download_include_embedded_videos.lower() == "true",
        )
    if email_notifications_album_invite is not None:
        set_nested(
            json_data,
            ["email_notifications_album_invite"],
            email_notifications_album_invite.lower() == "true",
        )
    if email_notifications_album_update is not None:
        set_nested(
            json_data,
            ["email_notifications_album_update"],
            email_notifications_album_update.lower() == "true",
        )
    if email_notifications_enabled is not None:
        set_nested(
            json_data,
            ["email_notifications_enabled"],
            email_notifications_enabled.lower() == "true",
        )
    if folders_enabled is not None:
        set_nested(json_data, ["folders_enabled"], folders_enabled.lower() == "true")
    if folders_sidebar_web is not None:
        set_nested(
            json_data, ["folders_sidebar_web"], folders_sidebar_web.lower() == "true"
        )
    if memories_duration is not None:
        set_nested(json_data, ["memories_duration"], memories_duration)
    if memories_enabled is not None:
        set_nested(json_data, ["memories_enabled"], memories_enabled.lower() == "true")
    if people_enabled is not None:
        set_nested(json_data, ["people_enabled"], people_enabled.lower() == "true")
    if people_sidebar_web is not None:
        set_nested(
            json_data, ["people_sidebar_web"], people_sidebar_web.lower() == "true"
        )
    if purchase_hide_buy_button_until is not None:
        set_nested(
            json_data,
            ["purchase_hide_buy_button_until"],
            purchase_hide_buy_button_until,
        )
    if purchase_show_support_badge is not None:
        set_nested(
            json_data,
            ["purchase_show_support_badge"],
            purchase_show_support_badge.lower() == "true",
        )
    if ratings_enabled is not None:
        set_nested(json_data, ["ratings_enabled"], ratings_enabled.lower() == "true")
    if shared_links_enabled is not None:
        set_nested(
            json_data, ["shared_links_enabled"], shared_links_enabled.lower() == "true"
        )
    if shared_links_sidebar_web is not None:
        set_nested(
            json_data,
            ["shared_links_sidebar_web"],
            shared_links_sidebar_web.lower() == "true",
        )
    if tags_enabled is not None:
        set_nested(json_data, ["tags_enabled"], tags_enabled.lower() == "true")
    if tags_sidebar_web is not None:
        set_nested(json_data, ["tags_sidebar_web"], tags_sidebar_web.lower() == "true")
    user_preferences_update_dto = UserPreferencesUpdateDto.model_validate(json_data)
    kwargs["user_preferences_update_dto"] = user_preferences_update_dto
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.users, "update_my_preferences", ctx, **kwargs)
    print_response(result, ctx)


@app.command("update-my-user", deprecated=False, rich_help_panel="API commands")
def update_my_user(
    ctx: typer.Context,
    avatar_color: str | None = typer.Option(None, "--avatar-color", help=""""""),
    email: str | None = typer.Option(None, "--email", help=""""""),
    name: str | None = typer.Option(None, "--name", help=""""""),
    password: str | None = typer.Option(None, "--password", help=""""""),
) -> None:
    """Update current user

    [link=https://api.immich.app/endpoints/users/updateMyUser]Immich API documentation[/link]
    """
    kwargs = {}
    json_data = {}
    if avatar_color is not None:
        set_nested(json_data, ["avatar_color"], avatar_color)
    if email is not None:
        set_nested(json_data, ["email"], email)
    if name is not None:
        set_nested(json_data, ["name"], name)
    if password is not None:
        set_nested(json_data, ["password"], password)
    user_update_me_dto = UserUpdateMeDto.model_validate(json_data)
    kwargs["user_update_me_dto"] = user_update_me_dto
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.users, "update_my_user", ctx, **kwargs)
    print_response(result, ctx)



"""Generated CLI commands for Plugins tag (auto-generated, do not edit)."""

from __future__ import annotations

import typer
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from immich import AsyncClient

from immich.cli.runtime import print_response, run_command
from immich.client.generated.models import *

app = typer.Typer(
    help="""A plugin is an installed module that makes filters and actions available for the workflow feature.\n\n[link=https://api.immich.app/endpoints/plugins]Immich API documentation[/link]"""
)


@app.command("get-plugin", deprecated=False, rich_help_panel="API commands")
def get_plugin(
    ctx: typer.Context,
    id: str = typer.Argument(..., help=""""""),
) -> None:
    """Retrieve a plugin

    [link=https://api.immich.app/endpoints/plugins/getPlugin]Immich API documentation[/link]
    """
    kwargs = {}
    kwargs["id"] = id
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.plugins, "get_plugin", ctx, **kwargs)
    print_response(result, ctx)


@app.command("get-plugin-triggers", deprecated=False, rich_help_panel="API commands")
def get_plugin_triggers(
    ctx: typer.Context,
) -> None:
    """List all plugin triggers

    [link=https://api.immich.app/endpoints/plugins/getPluginTriggers]Immich API documentation[/link]
    """
    kwargs = {}
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.plugins, "get_plugin_triggers", ctx, **kwargs)
    print_response(result, ctx)


@app.command("get-plugins", deprecated=False, rich_help_panel="API commands")
def get_plugins(
    ctx: typer.Context,
) -> None:
    """List all plugins

    [link=https://api.immich.app/endpoints/plugins/getPlugins]Immich API documentation[/link]
    """
    kwargs = {}
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.plugins, "get_plugins", ctx, **kwargs)
    print_response(result, ctx)



"""Generated CLI commands for Maintenance (admin) tag (auto-generated, do not edit)."""

from __future__ import annotations

import typer
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from immich import AsyncClient

from immich.cli.runtime import print_response, run_command, set_nested
from immich.client.generated.models import *

app = typer.Typer(
    help="""Maintenance mode allows you to put Immich in a read-only state to perform various operations.\n\n[link=https://api.immich.app/endpoints/maintenance-admin]Immich API documentation[/link]"""
)


@app.command("detect-prior-install", deprecated=False, rich_help_panel="API commands")
def detect_prior_install(
    ctx: typer.Context,
) -> None:
    """Detect existing install

    [link=https://api.immich.app/endpoints/maintenance-admin/detectPriorInstall]Immich API documentation[/link]
    """
    kwargs = {}
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(
        client, client.maintenance_admin, "detect_prior_install", ctx, **kwargs
    )
    print_response(result, ctx)


@app.command("get-maintenance-status", deprecated=False, rich_help_panel="API commands")
def get_maintenance_status(
    ctx: typer.Context,
) -> None:
    """Get maintenance mode status

    [link=https://api.immich.app/endpoints/maintenance-admin/getMaintenanceStatus]Immich API documentation[/link]
    """
    kwargs = {}
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(
        client, client.maintenance_admin, "get_maintenance_status", ctx, **kwargs
    )
    print_response(result, ctx)


@app.command("maintenance-login", deprecated=False, rich_help_panel="API commands")
def maintenance_login(
    ctx: typer.Context,
    token: str | None = typer.Option(None, "--token", help=""""""),
) -> None:
    """Log into maintenance mode

    [link=https://api.immich.app/endpoints/maintenance-admin/maintenanceLogin]Immich API documentation[/link]
    """
    kwargs = {}
    json_data = {}
    if token is not None:
        set_nested(json_data, ["token"], token)
    maintenance_login_dto = MaintenanceLoginDto.model_validate(json_data)
    kwargs["maintenance_login_dto"] = maintenance_login_dto
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(
        client, client.maintenance_admin, "maintenance_login", ctx, **kwargs
    )
    print_response(result, ctx)


@app.command("set-maintenance-mode", deprecated=False, rich_help_panel="API commands")
def set_maintenance_mode(
    ctx: typer.Context,
    action: str = typer.Option(..., "--action", help=""""""),
    restore_backup_filename: str | None = typer.Option(
        None, "--restore-backup-filename", help=""""""
    ),
) -> None:
    """Set maintenance mode

    [link=https://api.immich.app/endpoints/maintenance-admin/setMaintenanceMode]Immich API documentation[/link]
    """
    kwargs = {}
    json_data = {}
    set_nested(json_data, ["action"], action)
    if restore_backup_filename is not None:
        set_nested(json_data, ["restore_backup_filename"], restore_backup_filename)
    set_maintenance_mode_dto = SetMaintenanceModeDto.model_validate(json_data)
    kwargs["set_maintenance_mode_dto"] = set_maintenance_mode_dto
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(
        client, client.maintenance_admin, "set_maintenance_mode", ctx, **kwargs
    )
    print_response(result, ctx)



"""Generated CLI commands for Search tag (auto-generated, do not edit)."""

from __future__ import annotations

import typer
from datetime import datetime
from typing import Literal, TYPE_CHECKING

if TYPE_CHECKING:
    from immich import AsyncClient

from immich.cli.runtime import print_response, run_command, set_nested
from immich.client.generated.models import *

app = typer.Typer(
    help="""Endpoints related to searching assets via text, smart search, optical character recognition (OCR), and other filters like person, album, and other metadata. Search endpoints usually support pagination and sorting.\n\n[link=https://api.immich.app/endpoints/search]Immich API documentation[/link]"""
)


@app.command("get-assets-by-city", deprecated=False, rich_help_panel="API commands")
def get_assets_by_city(
    ctx: typer.Context,
) -> None:
    """Retrieve assets by city

    [link=https://api.immich.app/endpoints/search/getAssetsByCity]Immich API documentation[/link]
    """
    kwargs = {}
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.search, "get_assets_by_city", ctx, **kwargs)
    print_response(result, ctx)


@app.command("get-explore-data", deprecated=False, rich_help_panel="API commands")
def get_explore_data(
    ctx: typer.Context,
) -> None:
    """Retrieve explore data

    [link=https://api.immich.app/endpoints/search/getExploreData]Immich API documentation[/link]
    """
    kwargs = {}
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.search, "get_explore_data", ctx, **kwargs)
    print_response(result, ctx)


@app.command("get-search-suggestions", deprecated=False, rich_help_panel="API commands")
def get_search_suggestions(
    ctx: typer.Context,
    country: str | None = typer.Option(None, "--country", help=""""""),
    include_null: Literal["true", "false"] | None = typer.Option(
        None, "--include-null", help=""""""
    ),
    lens_model: str | None = typer.Option(None, "--lens-model", help=""""""),
    make: str | None = typer.Option(None, "--make", help=""""""),
    model: str | None = typer.Option(None, "--model", help=""""""),
    state: str | None = typer.Option(None, "--state", help=""""""),
    type: SearchSuggestionType = typer.Option(..., "--type", help=""""""),
) -> None:
    """Retrieve search suggestions

    [link=https://api.immich.app/endpoints/search/getSearchSuggestions]Immich API documentation[/link]
    """
    kwargs = {}
    if country is not None:
        kwargs["country"] = country
    if include_null is not None:
        kwargs["include_null"] = include_null.lower() == "true"
    if lens_model is not None:
        kwargs["lens_model"] = lens_model
    if make is not None:
        kwargs["make"] = make
    if model is not None:
        kwargs["model"] = model
    if state is not None:
        kwargs["state"] = state
    kwargs["type"] = type
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.search, "get_search_suggestions", ctx, **kwargs)
    print_response(result, ctx)


@app.command(
    "search-asset-statistics", deprecated=False, rich_help_panel="API commands"
)
def search_asset_statistics(
    ctx: typer.Context,
    album_ids: list[str] | None = typer.Option(None, "--album-ids", help=""""""),
    city: str | None = typer.Option(None, "--city", help=""""""),
    country: str | None = typer.Option(None, "--country", help=""""""),
    created_after: datetime | None = typer.Option(None, "--created-after", help=""""""),
    created_before: datetime | None = typer.Option(
        None, "--created-before", help=""""""
    ),
    description: str | None = typer.Option(None, "--description", help=""""""),
    device_id: str | None = typer.Option(None, "--device-id", help=""""""),
    is_encoded: Literal["true", "false"] | None = typer.Option(
        None, "--is-encoded", help=""""""
    ),
    is_favorite: Literal["true", "false"] | None = typer.Option(
        None, "--is-favorite", help=""""""
    ),
    is_motion: Literal["true", "false"] | None = typer.Option(
        None, "--is-motion", help=""""""
    ),
    is_not_in_album: Literal["true", "false"] | None = typer.Option(
        None, "--is-not-in-album", help=""""""
    ),
    is_offline: Literal["true", "false"] | None = typer.Option(
        None, "--is-offline", help=""""""
    ),
    lens_model: str | None = typer.Option(None, "--lens-model", help=""""""),
    library_id: str | None = typer.Option(None, "--library-id", help=""""""),
    make: str | None = typer.Option(None, "--make", help=""""""),
    model: str | None = typer.Option(None, "--model", help=""""""),
    ocr: str | None = typer.Option(None, "--ocr", help=""""""),
    person_ids: list[str] | None = typer.Option(None, "--person-ids", help=""""""),
    rating: float | None = typer.Option(None, "--rating", help="""""", min=-1, max=5),
    state: str | None = typer.Option(None, "--state", help=""""""),
    tag_ids: list[str] | None = typer.Option(None, "--tag-ids", help=""""""),
    taken_after: datetime | None = typer.Option(None, "--taken-after", help=""""""),
    taken_before: datetime | None = typer.Option(None, "--taken-before", help=""""""),
    trashed_after: datetime | None = typer.Option(None, "--trashed-after", help=""""""),
    trashed_before: datetime | None = typer.Option(
        None, "--trashed-before", help=""""""
    ),
    type: str | None = typer.Option(None, "--type", help=""""""),
    updated_after: datetime | None = typer.Option(None, "--updated-after", help=""""""),
    updated_before: datetime | None = typer.Option(
        None, "--updated-before", help=""""""
    ),
    visibility: str | None = typer.Option(None, "--visibility", help=""""""),
) -> None:
    """Search asset statistics

    [link=https://api.immich.app/endpoints/search/searchAssetStatistics]Immich API documentation[/link]
    """
    kwargs = {}
    json_data = {}
    if album_ids is not None:
        set_nested(json_data, ["album_ids"], album_ids)
    if city is not None:
        set_nested(json_data, ["city"], city)
    if country is not None:
        set_nested(json_data, ["country"], country)
    if created_after is not None:
        set_nested(json_data, ["created_after"], created_after)
    if created_before is not None:
        set_nested(json_data, ["created_before"], created_before)
    if description is not None:
        set_nested(json_data, ["description"], description)
    if device_id is not None:
        set_nested(json_data, ["device_id"], device_id)
    if is_encoded is not None:
        set_nested(json_data, ["is_encoded"], is_encoded.lower() == "true")
    if is_favorite is not None:
        set_nested(json_data, ["is_favorite"], is_favorite.lower() == "true")
    if is_motion is not None:
        set_nested(json_data, ["is_motion"], is_motion.lower() == "true")
    if is_not_in_album is not None:
        set_nested(json_data, ["is_not_in_album"], is_not_in_album.lower() == "true")
    if is_offline is not None:
        set_nested(json_data, ["is_offline"], is_offline.lower() == "true")
    if lens_model is not None:
        set_nested(json_data, ["lens_model"], lens_model)
    if library_id is not None:
        set_nested(json_data, ["library_id"], library_id)
    if make is not None:
        set_nested(json_data, ["make"], make)
    if model is not None:
        set_nested(json_data, ["model"], model)
    if ocr is not None:
        set_nested(json_data, ["ocr"], ocr)
    if person_ids is not None:
        set_nested(json_data, ["person_ids"], person_ids)
    if rating is not None:
        set_nested(json_data, ["rating"], rating)
    if state is not None:
        set_nested(json_data, ["state"], state)
    if tag_ids is not None:
        set_nested(json_data, ["tag_ids"], tag_ids)
    if taken_after is not None:
        set_nested(json_data, ["taken_after"], taken_after)
    if taken_before is not None:
        set_nested(json_data, ["taken_before"], taken_before)
    if trashed_after is not None:
        set_nested(json_data, ["trashed_after"], trashed_after)
    if trashed_before is not None:
        set_nested(json_data, ["trashed_before"], trashed_before)
    if type is not None:
        set_nested(json_data, ["type"], type)
    if updated_after is not None:
        set_nested(json_data, ["updated_after"], updated_after)
    if updated_before is not None:
        set_nested(json_data, ["updated_before"], updated_before)
    if visibility is not None:
        set_nested(json_data, ["visibility"], visibility)
    statistics_search_dto = StatisticsSearchDto.model_validate(json_data)
    kwargs["statistics_search_dto"] = statistics_search_dto
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(
        client, client.search, "search_asset_statistics", ctx, **kwargs
    )
    print_response(result, ctx)


@app.command("search-assets", deprecated=False, rich_help_panel="API commands")
def search_assets(
    ctx: typer.Context,
    album_ids: list[str] | None = typer.Option(None, "--album-ids", help=""""""),
    checksum: str | None = typer.Option(None, "--checksum", help=""""""),
    city: str | None = typer.Option(None, "--city", help=""""""),
    country: str | None = typer.Option(None, "--country", help=""""""),
    created_after: datetime | None = typer.Option(None, "--created-after", help=""""""),
    created_before: datetime | None = typer.Option(
        None, "--created-before", help=""""""
    ),
    description: str | None = typer.Option(None, "--description", help=""""""),
    device_asset_id: str | None = typer.Option(None, "--device-asset-id", help=""""""),
    device_id: str | None = typer.Option(None, "--device-id", help=""""""),
    encoded_video_path: str | None = typer.Option(
        None, "--encoded-video-path", help=""""""
    ),
    id: str | None = typer.Option(None, "--id", help=""""""),
    is_encoded: Literal["true", "false"] | None = typer.Option(
        None, "--is-encoded", help=""""""
    ),
    is_favorite: Literal["true", "false"] | None = typer.Option(
        None, "--is-favorite", help=""""""
    ),
    is_motion: Literal["true", "false"] | None = typer.Option(
        None, "--is-motion", help=""""""
    ),
    is_not_in_album: Literal["true", "false"] | None = typer.Option(
        None, "--is-not-in-album", help=""""""
    ),
    is_offline: Literal["true", "false"] | None = typer.Option(
        None, "--is-offline", help=""""""
    ),
    lens_model: str | None = typer.Option(None, "--lens-model", help=""""""),
    library_id: str | None = typer.Option(None, "--library-id", help=""""""),
    make: str | None = typer.Option(None, "--make", help=""""""),
    model: str | None = typer.Option(None, "--model", help=""""""),
    ocr: str | None = typer.Option(None, "--ocr", help=""""""),
    order: str | None = typer.Option(None, "--order", help=""""""),
    original_file_name: str | None = typer.Option(
        None, "--original-file-name", help=""""""
    ),
    original_path: str | None = typer.Option(None, "--original-path", help=""""""),
    page: float | None = typer.Option(None, "--page", help="""""", min=1),
    person_ids: list[str] | None = typer.Option(None, "--person-ids", help=""""""),
    preview_path: str | None = typer.Option(None, "--preview-path", help=""""""),
    rating: float | None = typer.Option(None, "--rating", help="""""", min=-1, max=5),
    size: float | None = typer.Option(None, "--size", help="""""", min=1, max=1000),
    state: str | None = typer.Option(None, "--state", help=""""""),
    tag_ids: list[str] | None = typer.Option(None, "--tag-ids", help=""""""),
    taken_after: datetime | None = typer.Option(None, "--taken-after", help=""""""),
    taken_before: datetime | None = typer.Option(None, "--taken-before", help=""""""),
    thumbnail_path: str | None = typer.Option(None, "--thumbnail-path", help=""""""),
    trashed_after: datetime | None = typer.Option(None, "--trashed-after", help=""""""),
    trashed_before: datetime | None = typer.Option(
        None, "--trashed-before", help=""""""
    ),
    type: str | None = typer.Option(None, "--type", help=""""""),
    updated_after: datetime | None = typer.Option(None, "--updated-after", help=""""""),
    updated_before: datetime | None = typer.Option(
        None, "--updated-before", help=""""""
    ),
    visibility: str | None = typer.Option(None, "--visibility", help=""""""),
    with_deleted: Literal["true", "false"] | None = typer.Option(
        None, "--with-deleted", help=""""""
    ),
    with_exif: Literal["true", "false"] | None = typer.Option(
        None, "--with-exif", help=""""""
    ),
    with_people: Literal["true", "false"] | None = typer.Option(
        None, "--with-people", help=""""""
    ),
    with_stacked: Literal["true", "false"] | None = typer.Option(
        None, "--with-stacked", help=""""""
    ),
) -> None:
    """Search assets by metadata

    [link=https://api.immich.app/endpoints/search/searchAssets]Immich API documentation[/link]
    """
    kwargs = {}
    json_data = {}
    if album_ids is not None:
        set_nested(json_data, ["album_ids"], album_ids)
    if checksum is not None:
        set_nested(json_data, ["checksum"], checksum)
    if city is not None:
        set_nested(json_data, ["city"], city)
    if country is not None:
        set_nested(json_data, ["country"], country)
    if created_after is not None:
        set_nested(json_data, ["created_after"], created_after)
    if created_before is not None:
        set_nested(json_data, ["created_before"], created_before)
    if description is not None:
        set_nested(json_data, ["description"], description)
    if device_asset_id is not None:
        set_nested(json_data, ["device_asset_id"], device_asset_id)
    if device_id is not None:
        set_nested(json_data, ["device_id"], device_id)
    if encoded_video_path is not None:
        set_nested(json_data, ["encoded_video_path"], encoded_video_path)
    if id is not None:
        set_nested(json_data, ["id"], id)
    if is_encoded is not None:
        set_nested(json_data, ["is_encoded"], is_encoded.lower() == "true")
    if is_favorite is not None:
        set_nested(json_data, ["is_favorite"], is_favorite.lower() == "true")
    if is_motion is not None:
        set_nested(json_data, ["is_motion"], is_motion.lower() == "true")
    if is_not_in_album is not None:
        set_nested(json_data, ["is_not_in_album"], is_not_in_album.lower() == "true")
    if is_offline is not None:
        set_nested(json_data, ["is_offline"], is_offline.lower() == "true")
    if lens_model is not None:
        set_nested(json_data, ["lens_model"], lens_model)
    if library_id is not None:
        set_nested(json_data, ["library_id"], library_id)
    if make is not None:
        set_nested(json_data, ["make"], make)
    if model is not None:
        set_nested(json_data, ["model"], model)
    if ocr is not None:
        set_nested(json_data, ["ocr"], ocr)
    if order is not None:
        set_nested(json_data, ["order"], order)
    if original_file_name is not None:
        set_nested(json_data, ["original_file_name"], original_file_name)
    if original_path is not None:
        set_nested(json_data, ["original_path"], original_path)
    if page is not None:
        set_nested(json_data, ["page"], page)
    if person_ids is not None:
        set_nested(json_data, ["person_ids"], person_ids)
    if preview_path is not None:
        set_nested(json_data, ["preview_path"], preview_path)
    if rating is not None:
        set_nested(json_data, ["rating"], rating)
    if size is not None:
        set_nested(json_data, ["size"], size)
    if state is not None:
        set_nested(json_data, ["state"], state)
    if tag_ids is not None:
        set_nested(json_data, ["tag_ids"], tag_ids)
    if taken_after is not None:
        set_nested(json_data, ["taken_after"], taken_after)
    if taken_before is not None:
        set_nested(json_data, ["taken_before"], taken_before)
    if thumbnail_path is not None:
        set_nested(json_data, ["thumbnail_path"], thumbnail_path)
    if trashed_after is not None:
        set_nested(json_data, ["trashed_after"], trashed_after)
    if trashed_before is not None:
        set_nested(json_data, ["trashed_before"], trashed_before)
    if type is not None:
        set_nested(json_data, ["type"], type)
    if updated_after is not None:
        set_nested(json_data, ["updated_after"], updated_after)
    if updated_before is not None:
        set_nested(json_data, ["updated_before"], updated_before)
    if visibility is not None:
        set_nested(json_data, ["visibility"], visibility)
    if with_deleted is not None:
        set_nested(json_data, ["with_deleted"], with_deleted.lower() == "true")
    if with_exif is not None:
        set_nested(json_data, ["with_exif"], with_exif.lower() == "true")
    if with_people is not None:
        set_nested(json_data, ["with_people"], with_people.lower() == "true")
    if with_stacked is not None:
        set_nested(json_data, ["with_stacked"], with_stacked.lower() == "true")
    metadata_search_dto = MetadataSearchDto.model_validate(json_data)
    kwargs["metadata_search_dto"] = metadata_search_dto
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.search, "search_assets", ctx, **kwargs)
    print_response(result, ctx)


@app.command("search-large-assets", deprecated=False, rich_help_panel="API commands")
def search_large_assets(
    ctx: typer.Context,
    album_ids: list[str] | None = typer.Option(None, "--album-ids", help=""""""),
    city: str | None = typer.Option(None, "--city", help=""""""),
    country: str | None = typer.Option(None, "--country", help=""""""),
    created_after: datetime | None = typer.Option(None, "--created-after", help=""""""),
    created_before: datetime | None = typer.Option(
        None, "--created-before", help=""""""
    ),
    device_id: str | None = typer.Option(None, "--device-id", help=""""""),
    is_encoded: Literal["true", "false"] | None = typer.Option(
        None, "--is-encoded", help=""""""
    ),
    is_favorite: Literal["true", "false"] | None = typer.Option(
        None, "--is-favorite", help=""""""
    ),
    is_motion: Literal["true", "false"] | None = typer.Option(
        None, "--is-motion", help=""""""
    ),
    is_not_in_album: Literal["true", "false"] | None = typer.Option(
        None, "--is-not-in-album", help=""""""
    ),
    is_offline: Literal["true", "false"] | None = typer.Option(
        None, "--is-offline", help=""""""
    ),
    lens_model: str | None = typer.Option(None, "--lens-model", help=""""""),
    library_id: str | None = typer.Option(None, "--library-id", help=""""""),
    make: str | None = typer.Option(None, "--make", help=""""""),
    min_file_size: int | None = typer.Option(
        None, "--min-file-size", help="""""", min=0
    ),
    model: str | None = typer.Option(None, "--model", help=""""""),
    ocr: str | None = typer.Option(None, "--ocr", help=""""""),
    person_ids: list[str] | None = typer.Option(None, "--person-ids", help=""""""),
    rating: float | None = typer.Option(None, "--rating", help="""""", min=-1, max=5),
    size: float | None = typer.Option(None, "--size", help="""""", min=1, max=1000),
    state: str | None = typer.Option(None, "--state", help=""""""),
    tag_ids: list[str] | None = typer.Option(None, "--tag-ids", help=""""""),
    taken_after: datetime | None = typer.Option(None, "--taken-after", help=""""""),
    taken_before: datetime | None = typer.Option(None, "--taken-before", help=""""""),
    trashed_after: datetime | None = typer.Option(None, "--trashed-after", help=""""""),
    trashed_before: datetime | None = typer.Option(
        None, "--trashed-before", help=""""""
    ),
    type: AssetTypeEnum | None = typer.Option(None, "--type", help=""""""),
    updated_after: datetime | None = typer.Option(None, "--updated-after", help=""""""),
    updated_before: datetime | None = typer.Option(
        None, "--updated-before", help=""""""
    ),
    visibility: AssetVisibility | None = typer.Option(
        None, "--visibility", help=""""""
    ),
    with_deleted: Literal["true", "false"] | None = typer.Option(
        None, "--with-deleted", help=""""""
    ),
    with_exif: Literal["true", "false"] | None = typer.Option(
        None, "--with-exif", help=""""""
    ),
) -> None:
    """Search large assets

    [link=https://api.immich.app/endpoints/search/searchLargeAssets]Immich API documentation[/link]
    """
    kwargs = {}
    if album_ids is not None:
        kwargs["album_ids"] = album_ids
    if city is not None:
        kwargs["city"] = city
    if country is not None:
        kwargs["country"] = country
    if created_after is not None:
        kwargs["created_after"] = created_after
    if created_before is not None:
        kwargs["created_before"] = created_before
    if device_id is not None:
        kwargs["device_id"] = device_id
    if is_encoded is not None:
        kwargs["is_encoded"] = is_encoded.lower() == "true"
    if is_favorite is not None:
        kwargs["is_favorite"] = is_favorite.lower() == "true"
    if is_motion is not None:
        kwargs["is_motion"] = is_motion.lower() == "true"
    if is_not_in_album is not None:
        kwargs["is_not_in_album"] = is_not_in_album.lower() == "true"
    if is_offline is not None:
        kwargs["is_offline"] = is_offline.lower() == "true"
    if lens_model is not None:
        kwargs["lens_model"] = lens_model
    if library_id is not None:
        kwargs["library_id"] = library_id
    if make is not None:
        kwargs["make"] = make
    if min_file_size is not None:
        kwargs["min_file_size"] = min_file_size
    if model is not None:
        kwargs["model"] = model
    if ocr is not None:
        kwargs["ocr"] = ocr
    if person_ids is not None:
        kwargs["person_ids"] = person_ids
    if rating is not None:
        kwargs["rating"] = rating
    if size is not None:
        kwargs["size"] = size
    if state is not None:
        kwargs["state"] = state
    if tag_ids is not None:
        kwargs["tag_ids"] = tag_ids
    if taken_after is not None:
        kwargs["taken_after"] = taken_after
    if taken_before is not None:
        kwargs["taken_before"] = taken_before
    if trashed_after is not None:
        kwargs["trashed_after"] = trashed_after
    if trashed_before is not None:
        kwargs["trashed_before"] = trashed_before
    if type is not None:
        kwargs["type"] = type
    if updated_after is not None:
        kwargs["updated_after"] = updated_after
    if updated_before is not None:
        kwargs["updated_before"] = updated_before
    if visibility is not None:
        kwargs["visibility"] = visibility
    if with_deleted is not None:
        kwargs["with_deleted"] = with_deleted.lower() == "true"
    if with_exif is not None:
        kwargs["with_exif"] = with_exif.lower() == "true"
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.search, "search_large_assets", ctx, **kwargs)
    print_response(result, ctx)


@app.command("search-person", deprecated=False, rich_help_panel="API commands")
def search_person(
    ctx: typer.Context,
    name: str = typer.Option(..., "--name", help=""""""),
    with_hidden: Literal["true", "false"] | None = typer.Option(
        None, "--with-hidden", help=""""""
    ),
) -> None:
    """Search people

    [link=https://api.immich.app/endpoints/search/searchPerson]Immich API documentation[/link]
    """
    kwargs = {}
    kwargs["name"] = name
    if with_hidden is not None:
        kwargs["with_hidden"] = with_hidden.lower() == "true"
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.search, "search_person", ctx, **kwargs)
    print_response(result, ctx)


@app.command("search-places", deprecated=False, rich_help_panel="API commands")
def search_places(
    ctx: typer.Context,
    name: str = typer.Option(..., "--name", help=""""""),
) -> None:
    """Search places

    [link=https://api.immich.app/endpoints/search/searchPlaces]Immich API documentation[/link]
    """
    kwargs = {}
    kwargs["name"] = name
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.search, "search_places", ctx, **kwargs)
    print_response(result, ctx)


@app.command("search-random", deprecated=False, rich_help_panel="API commands")
def search_random(
    ctx: typer.Context,
    album_ids: list[str] | None = typer.Option(None, "--album-ids", help=""""""),
    city: str | None = typer.Option(None, "--city", help=""""""),
    country: str | None = typer.Option(None, "--country", help=""""""),
    created_after: datetime | None = typer.Option(None, "--created-after", help=""""""),
    created_before: datetime | None = typer.Option(
        None, "--created-before", help=""""""
    ),
    device_id: str | None = typer.Option(None, "--device-id", help=""""""),
    is_encoded: Literal["true", "false"] | None = typer.Option(
        None, "--is-encoded", help=""""""
    ),
    is_favorite: Literal["true", "false"] | None = typer.Option(
        None, "--is-favorite", help=""""""
    ),
    is_motion: Literal["true", "false"] | None = typer.Option(
        None, "--is-motion", help=""""""
    ),
    is_not_in_album: Literal["true", "false"] | None = typer.Option(
        None, "--is-not-in-album", help=""""""
    ),
    is_offline: Literal["true", "false"] | None = typer.Option(
        None, "--is-offline", help=""""""
    ),
    lens_model: str | None = typer.Option(None, "--lens-model", help=""""""),
    library_id: str | None = typer.Option(None, "--library-id", help=""""""),
    make: str | None = typer.Option(None, "--make", help=""""""),
    model: str | None = typer.Option(None, "--model", help=""""""),
    ocr: str | None = typer.Option(None, "--ocr", help=""""""),
    person_ids: list[str] | None = typer.Option(None, "--person-ids", help=""""""),
    rating: float | None = typer.Option(None, "--rating", help="""""", min=-1, max=5),
    size: float | None = typer.Option(None, "--size", help="""""", min=1, max=1000),
    state: str | None = typer.Option(None, "--state", help=""""""),
    tag_ids: list[str] | None = typer.Option(None, "--tag-ids", help=""""""),
    taken_after: datetime | None = typer.Option(None, "--taken-after", help=""""""),
    taken_before: datetime | None = typer.Option(None, "--taken-before", help=""""""),
    trashed_after: datetime | None = typer.Option(None, "--trashed-after", help=""""""),
    trashed_before: datetime | None = typer.Option(
        None, "--trashed-before", help=""""""
    ),
    type: str | None = typer.Option(None, "--type", help=""""""),
    updated_after: datetime | None = typer.Option(None, "--updated-after", help=""""""),
    updated_before: datetime | None = typer.Option(
        None, "--updated-before", help=""""""
    ),
    visibility: str | None = typer.Option(None, "--visibility", help=""""""),
    with_deleted: Literal["true", "false"] | None = typer.Option(
        None, "--with-deleted", help=""""""
    ),
    with_exif: Literal["true", "false"] | None = typer.Option(
        None, "--with-exif", help=""""""
    ),
    with_people: Literal["true", "false"] | None = typer.Option(
        None, "--with-people", help=""""""
    ),
    with_stacked: Literal["true", "false"] | None = typer.Option(
        None, "--with-stacked", help=""""""
    ),
) -> None:
    """Search random assets

    [link=https://api.immich.app/endpoints/search/searchRandom]Immich API documentation[/link]
    """
    kwargs = {}
    json_data = {}
    if album_ids is not None:
        set_nested(json_data, ["album_ids"], album_ids)
    if city is not None:
        set_nested(json_data, ["city"], city)
    if country is not None:
        set_nested(json_data, ["country"], country)
    if created_after is not None:
        set_nested(json_data, ["created_after"], created_after)
    if created_before is not None:
        set_nested(json_data, ["created_before"], created_before)
    if device_id is not None:
        set_nested(json_data, ["device_id"], device_id)
    if is_encoded is not None:
        set_nested(json_data, ["is_encoded"], is_encoded.lower() == "true")
    if is_favorite is not None:
        set_nested(json_data, ["is_favorite"], is_favorite.lower() == "true")
    if is_motion is not None:
        set_nested(json_data, ["is_motion"], is_motion.lower() == "true")
    if is_not_in_album is not None:
        set_nested(json_data, ["is_not_in_album"], is_not_in_album.lower() == "true")
    if is_offline is not None:
        set_nested(json_data, ["is_offline"], is_offline.lower() == "true")
    if lens_model is not None:
        set_nested(json_data, ["lens_model"], lens_model)
    if library_id is not None:
        set_nested(json_data, ["library_id"], library_id)
    if make is not None:
        set_nested(json_data, ["make"], make)
    if model is not None:
        set_nested(json_data, ["model"], model)
    if ocr is not None:
        set_nested(json_data, ["ocr"], ocr)
    if person_ids is not None:
        set_nested(json_data, ["person_ids"], person_ids)
    if rating is not None:
        set_nested(json_data, ["rating"], rating)
    if size is not None:
        set_nested(json_data, ["size"], size)
    if state is not None:
        set_nested(json_data, ["state"], state)
    if tag_ids is not None:
        set_nested(json_data, ["tag_ids"], tag_ids)
    if taken_after is not None:
        set_nested(json_data, ["taken_after"], taken_after)
    if taken_before is not None:
        set_nested(json_data, ["taken_before"], taken_before)
    if trashed_after is not None:
        set_nested(json_data, ["trashed_after"], trashed_after)
    if trashed_before is not None:
        set_nested(json_data, ["trashed_before"], trashed_before)
    if type is not None:
        set_nested(json_data, ["type"], type)
    if updated_after is not None:
        set_nested(json_data, ["updated_after"], updated_after)
    if updated_before is not None:
        set_nested(json_data, ["updated_before"], updated_before)
    if visibility is not None:
        set_nested(json_data, ["visibility"], visibility)
    if with_deleted is not None:
        set_nested(json_data, ["with_deleted"], with_deleted.lower() == "true")
    if with_exif is not None:
        set_nested(json_data, ["with_exif"], with_exif.lower() == "true")
    if with_people is not None:
        set_nested(json_data, ["with_people"], with_people.lower() == "true")
    if with_stacked is not None:
        set_nested(json_data, ["with_stacked"], with_stacked.lower() == "true")
    random_search_dto = RandomSearchDto.model_validate(json_data)
    kwargs["random_search_dto"] = random_search_dto
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.search, "search_random", ctx, **kwargs)
    print_response(result, ctx)


@app.command("search-smart", deprecated=False, rich_help_panel="API commands")
def search_smart(
    ctx: typer.Context,
    album_ids: list[str] | None = typer.Option(None, "--album-ids", help=""""""),
    city: str | None = typer.Option(None, "--city", help=""""""),
    country: str | None = typer.Option(None, "--country", help=""""""),
    created_after: datetime | None = typer.Option(None, "--created-after", help=""""""),
    created_before: datetime | None = typer.Option(
        None, "--created-before", help=""""""
    ),
    device_id: str | None = typer.Option(None, "--device-id", help=""""""),
    is_encoded: Literal["true", "false"] | None = typer.Option(
        None, "--is-encoded", help=""""""
    ),
    is_favorite: Literal["true", "false"] | None = typer.Option(
        None, "--is-favorite", help=""""""
    ),
    is_motion: Literal["true", "false"] | None = typer.Option(
        None, "--is-motion", help=""""""
    ),
    is_not_in_album: Literal["true", "false"] | None = typer.Option(
        None, "--is-not-in-album", help=""""""
    ),
    is_offline: Literal["true", "false"] | None = typer.Option(
        None, "--is-offline", help=""""""
    ),
    language: str | None = typer.Option(None, "--language", help=""""""),
    lens_model: str | None = typer.Option(None, "--lens-model", help=""""""),
    library_id: str | None = typer.Option(None, "--library-id", help=""""""),
    make: str | None = typer.Option(None, "--make", help=""""""),
    model: str | None = typer.Option(None, "--model", help=""""""),
    ocr: str | None = typer.Option(None, "--ocr", help=""""""),
    page: float | None = typer.Option(None, "--page", help="""""", min=1),
    person_ids: list[str] | None = typer.Option(None, "--person-ids", help=""""""),
    query: str | None = typer.Option(None, "--query", help=""""""),
    query_asset_id: str | None = typer.Option(None, "--query-asset-id", help=""""""),
    rating: float | None = typer.Option(None, "--rating", help="""""", min=-1, max=5),
    size: float | None = typer.Option(None, "--size", help="""""", min=1, max=1000),
    state: str | None = typer.Option(None, "--state", help=""""""),
    tag_ids: list[str] | None = typer.Option(None, "--tag-ids", help=""""""),
    taken_after: datetime | None = typer.Option(None, "--taken-after", help=""""""),
    taken_before: datetime | None = typer.Option(None, "--taken-before", help=""""""),
    trashed_after: datetime | None = typer.Option(None, "--trashed-after", help=""""""),
    trashed_before: datetime | None = typer.Option(
        None, "--trashed-before", help=""""""
    ),
    type: str | None = typer.Option(None, "--type", help=""""""),
    updated_after: datetime | None = typer.Option(None, "--updated-after", help=""""""),
    updated_before: datetime | None = typer.Option(
        None, "--updated-before", help=""""""
    ),
    visibility: str | None = typer.Option(None, "--visibility", help=""""""),
    with_deleted: Literal["true", "false"] | None = typer.Option(
        None, "--with-deleted", help=""""""
    ),
    with_exif: Literal["true", "false"] | None = typer.Option(
        None, "--with-exif", help=""""""
    ),
) -> None:
    """Smart asset search

    [link=https://api.immich.app/endpoints/search/searchSmart]Immich API documentation[/link]
    """
    kwargs = {}
    json_data = {}
    if album_ids is not None:
        set_nested(json_data, ["album_ids"], album_ids)
    if city is not None:
        set_nested(json_data, ["city"], city)
    if country is not None:
        set_nested(json_data, ["country"], country)
    if created_after is not None:
        set_nested(json_data, ["created_after"], created_after)
    if created_before is not None:
        set_nested(json_data, ["created_before"], created_before)
    if device_id is not None:
        set_nested(json_data, ["device_id"], device_id)
    if is_encoded is not None:
        set_nested(json_data, ["is_encoded"], is_encoded.lower() == "true")
    if is_favorite is not None:
        set_nested(json_data, ["is_favorite"], is_favorite.lower() == "true")
    if is_motion is not None:
        set_nested(json_data, ["is_motion"], is_motion.lower() == "true")
    if is_not_in_album is not None:
        set_nested(json_data, ["is_not_in_album"], is_not_in_album.lower() == "true")
    if is_offline is not None:
        set_nested(json_data, ["is_offline"], is_offline.lower() == "true")
    if language is not None:
        set_nested(json_data, ["language"], language)
    if lens_model is not None:
        set_nested(json_data, ["lens_model"], lens_model)
    if library_id is not None:
        set_nested(json_data, ["library_id"], library_id)
    if make is not None:
        set_nested(json_data, ["make"], make)
    if model is not None:
        set_nested(json_data, ["model"], model)
    if ocr is not None:
        set_nested(json_data, ["ocr"], ocr)
    if page is not None:
        set_nested(json_data, ["page"], page)
    if person_ids is not None:
        set_nested(json_data, ["person_ids"], person_ids)
    if query is not None:
        set_nested(json_data, ["query"], query)
    if query_asset_id is not None:
        set_nested(json_data, ["query_asset_id"], query_asset_id)
    if rating is not None:
        set_nested(json_data, ["rating"], rating)
    if size is not None:
        set_nested(json_data, ["size"], size)
    if state is not None:
        set_nested(json_data, ["state"], state)
    if tag_ids is not None:
        set_nested(json_data, ["tag_ids"], tag_ids)
    if taken_after is not None:
        set_nested(json_data, ["taken_after"], taken_after)
    if taken_before is not None:
        set_nested(json_data, ["taken_before"], taken_before)
    if trashed_after is not None:
        set_nested(json_data, ["trashed_after"], trashed_after)
    if trashed_before is not None:
        set_nested(json_data, ["trashed_before"], trashed_before)
    if type is not None:
        set_nested(json_data, ["type"], type)
    if updated_after is not None:
        set_nested(json_data, ["updated_after"], updated_after)
    if updated_before is not None:
        set_nested(json_data, ["updated_before"], updated_before)
    if visibility is not None:
        set_nested(json_data, ["visibility"], visibility)
    if with_deleted is not None:
        set_nested(json_data, ["with_deleted"], with_deleted.lower() == "true")
    if with_exif is not None:
        set_nested(json_data, ["with_exif"], with_exif.lower() == "true")
    smart_search_dto = SmartSearchDto.model_validate(json_data)
    kwargs["smart_search_dto"] = smart_search_dto
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.search, "search_smart", ctx, **kwargs)
    print_response(result, ctx)



"""Generated CLI commands for API keys tag (auto-generated, do not edit)."""

from __future__ import annotations

import typer
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from immich import AsyncClient

from immich.cli.runtime import print_response, run_command, set_nested
from immich.client.generated.models import *

app = typer.Typer(
    help="""An api key can be used to programmatically access the Immich API.\n\n[link=https://api.immich.app/endpoints/api-keys]Immich API documentation[/link]"""
)


@app.command("create-api-key", deprecated=False, rich_help_panel="API commands")
def create_api_key(
    ctx: typer.Context,
    name: str | None = typer.Option(None, "--name", help=""""""),
    permissions: list[Permission] = typer.Option(..., "--permissions", help=""""""),
) -> None:
    """Create an API key

    [link=https://api.immich.app/endpoints/api-keys/createApiKey]Immich API documentation[/link]
    """
    kwargs = {}
    json_data = {}
    if name is not None:
        set_nested(json_data, ["name"], name)
    set_nested(json_data, ["permissions"], permissions)
    api_key_create_dto = APIKeyCreateDto.model_validate(json_data)
    kwargs["api_key_create_dto"] = api_key_create_dto
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.api_keys, "create_api_key", ctx, **kwargs)
    print_response(result, ctx)


@app.command("delete-api-key", deprecated=False, rich_help_panel="API commands")
def delete_api_key(
    ctx: typer.Context,
    id: str = typer.Argument(..., help=""""""),
) -> None:
    """Delete an API key

    [link=https://api.immich.app/endpoints/api-keys/deleteApiKey]Immich API documentation[/link]
    """
    kwargs = {}
    kwargs["id"] = id
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.api_keys, "delete_api_key", ctx, **kwargs)
    print_response(result, ctx)


@app.command("get-api-key", deprecated=False, rich_help_panel="API commands")
def get_api_key(
    ctx: typer.Context,
    id: str = typer.Argument(..., help=""""""),
) -> None:
    """Retrieve an API key

    [link=https://api.immich.app/endpoints/api-keys/getApiKey]Immich API documentation[/link]
    """
    kwargs = {}
    kwargs["id"] = id
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.api_keys, "get_api_key", ctx, **kwargs)
    print_response(result, ctx)


@app.command("get-api-keys", deprecated=False, rich_help_panel="API commands")
def get_api_keys(
    ctx: typer.Context,
) -> None:
    """List all API keys

    [link=https://api.immich.app/endpoints/api-keys/getApiKeys]Immich API documentation[/link]
    """
    kwargs = {}
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.api_keys, "get_api_keys", ctx, **kwargs)
    print_response(result, ctx)


@app.command("get-my-api-key", deprecated=False, rich_help_panel="API commands")
def get_my_api_key(
    ctx: typer.Context,
) -> None:
    """Retrieve the current API key

    [link=https://api.immich.app/endpoints/api-keys/getMyApiKey]Immich API documentation[/link]
    """
    kwargs = {}
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.api_keys, "get_my_api_key", ctx, **kwargs)
    print_response(result, ctx)


@app.command("update-api-key", deprecated=False, rich_help_panel="API commands")
def update_api_key(
    ctx: typer.Context,
    id: str = typer.Argument(..., help=""""""),
    name: str | None = typer.Option(None, "--name", help=""""""),
    permissions: list[Permission] | None = typer.Option(
        None, "--permissions", help=""""""
    ),
) -> None:
    """Update an API key

    [link=https://api.immich.app/endpoints/api-keys/updateApiKey]Immich API documentation[/link]
    """
    kwargs = {}
    json_data = {}
    kwargs["id"] = id
    if name is not None:
        set_nested(json_data, ["name"], name)
    if permissions is not None:
        set_nested(json_data, ["permissions"], permissions)
    api_key_update_dto = APIKeyUpdateDto.model_validate(json_data)
    kwargs["api_key_update_dto"] = api_key_update_dto
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.api_keys, "update_api_key", ctx, **kwargs)
    print_response(result, ctx)
